#!/usr/bin/perl

#┌─────────────────────────────────
#│ CLIP BOARD : clip.cgi - 2022/05/28
#│ copyright (c) kentweb, 1997-2022
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  ClipBoard v6.21 com-multi v2.0 : clip.cgi - 2023/10/17
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# アクセス制限
&passwd(%in) if ($cf{enter_pwd} ne '');

# 処理分岐
if ($in{mode} eq 'find') { find_data(); }
if ($in{mode} eq 'note') { note_page(); }
if ($in{mode} eq 'view') { view_page(); }
if ($in{mode} eq 'form') { form_page(); }
if ($in{mode} eq 'p_form') { p_form(); }
if ($in{mode} eq 'headline') { headline_page(); }
if ($in{mode} eq 'rescom') { rescom_form(); }
if ($in{mode} eq 'res_find') { res_find_data(); }
if ($in{mode} eq 'compwd') { com_pwdform(); }
if ($in{mode} eq 'expound') { expound_page(); }
if ($in{mode} eq 'res_album') { res_album(); }
if ($in{mode} eq 'lock') { lock_form(); }
if ($in{mode} eq 'past') { past_page(); }

bbs_list();

#-----------------------------------------------------------
#  記事リスト画面
#-----------------------------------------------------------
sub bbs_list {
	del_form() if ($in{del});

	# レスカウント
	my ($i,@log,%res_cnt,@reply,@main_log,@rescnt);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$date,$name,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# キーワード配列化
	$in{q} =~ s/　/ /g;
	my @q = split(/\s+/,$in{q});
	
	# ページ数定義
	my $pg = $in{pg} || 0;
	
	# 表示データ認識
	my ($i,@log,$imgt1,$imgt2,$imgt3,$imgt4,$imgt5,$imgt6);
	open(IN,"$cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);

			$imgt1 = $imgtxt1;
			$imgt2 = $imgtxt2;
			$imgt3 = $imgtxt3;
			$imgt4 = $imgtxt4;
			$imgt5 = $imgtxt5;
			$imgt6 = $imgtxt6;

		# --- 検索
		if ($in{q} ne '') {
			my $flg;
			for my $q (@q) {
				if ("$name $sub $com $imgt1 $imgt2 $imgt3 $imgt4 $imgt5 $imgt6" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$q\E/i) {
					$flg++;
				} else {
					$flg = 0;
					last;
				}
			}
			next if (!$flg);
		}

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{pg_max});
		
		push(@log,$_);
	}
	close(IN);
	
	# ページ繰越ボタン作成
	my $pgbtn = make_btn($i,$pg);
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/bbs.html") or error("open err: bbs.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置換
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="$1" class="icon">|g;
	$tmpl =~ s/!([a-z]+_cgi|homepage|cmnurl|bbs_title)!/$cf{$1}/g;
	$tmpl =~ s/!page_btn!/$pgbtn/g;
	$tmpl =~ s/!imgtext1!/$imgt1/g;
	$tmpl =~ s/!imgtext2!/$imgt2/g;
	$tmpl =~ s/!imgtext3!/$imgt3/g;
	$tmpl =~ s/!imgtext4!/$imgt4/g;
	$tmpl =~ s/!imgtext5!/$imgt5/g;
	$tmpl =~ s/!imgtext6!/$imgt6/g;
	$tmpl =~ s/!q!/$in{q}/g;

	if ($in{q} eq '') {
		$tmpl =~ s|<!-- find_result -->.+?<!-- /find_result -->||s;
	} else {
		$tmpl =~ s/!result!/$i/g;
	}

	# 過去ログ
	$tmpl =~ s|<!-- past -->.+?<!-- /past -->||s if ($cf{pastkey} == 0);
	
	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain,$str_crypt) = cap::make($cf{captcha_key},$cf{cap_len});
		$tmpl =~ s/!str_crypt!/$str_crypt/g;
	} else {
		$tmpl =~ s|<!-- captcha -->.+?<!-- /captcha -->||s;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error('テンプレート不正');
	
	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	# 表示データ展開
	foreach my $log (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/, $log);

		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{iconurl}/house04.png" class="icon" alt=""></a>|;
		$com = auto_link($com) if ($cf{autolink});
		
		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		# YouTube
		if ($cf{tube}) {
			$com =~ s|!([\w\-]{1,})!|<span class="imageCover"><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="$cf{cmnurl}/youtube.png" width="64" height="64" class="coverImage"></a></span>|g;
		}

		# 添付
		if (defined($ex1)) {
			$com = att_file($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1);
		}

		my $clip;
			$clip = attach($no,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$name,$sub,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

	my @col = split(/\s+/, $cf{colors});

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!email!/$eml/g;
		$tmp =~ s/!home!/$url/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!col2!/$col2/g;
		$tmp =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
		$tmp =~ s|!ico_edit!|<a href="$cf{bbs_cgi}?mode=p_form&no=$no"><img src="$cf{iconurl}/edit04.png" class="icon" title="編集" alt="" /></a>|g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!pg!/$pg/g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!ico_com!/$cf{ico_com}/g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;
		$tmp =~ s|!clip!|<img src="$cf{cmnurl}/paper_clip.png" border="0" alt="" class="img_a">|g;
		$tmp =~ s|!ico_lock!|<a href="$cf{bbs_cgi}?mode=lock&no=$_"><img src="$cf{cmnurl}/password.png" title="記事ロック" alt="記事ロック" class="icon" /></a>|g;

	# コメント返信機能ＯＦＦ
	if ($cf{comment} == 0) {
		$tmp =~ s/<!-- rescom_job_begin -->.+<!-- rescom_job_end -->//s;
	}

		if ($lock eq '1') {
			$tmp =~ s|<!-- lock_edit -->(.+?)<!-- /lock_edit -->||s;
			$tmp =~ s|<!-- lock_com -->(.+?)<!-- /lock_com -->||s;
		} elsif ($lock == 0 || $lock == '') {
			$tmp =~ s|<!-- lock -->(.+?)<!-- /lock -->||s;
		}

		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  記事ロックフォーム
#-----------------------------------------------------------
sub lock_form {

	$in{no} =~ s/\D//g;

	# 記事
	my ($name,$pwd);
	open(DAT,"$cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	my $lockey = <DAT>;
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);

		if ($in{no} == $no) {
			$name = $name;
			$sub = $sub;
			$pwd = $pw;
			last;
		}
	}
	close(DAT);

	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/,$lockey);

		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		$com = auto_link($com) if ($cf{autolink});

		#if (length($com) > 135) { $com = substr($com,0,140); $com .= '.....(略)'; }

	my @col = split(/\s+/, $cf{colors});

	open(IN,"$cf{tmpldir}/lock_form.html") or &error("open err: lock_form.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!ico:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/bbs.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!color2!/$col2/g;
	$tmpl =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;

	if ($lock == 1) {
		$tmpl =~ s|!lock_com!||g;
		$tmpl =~ s|!open_com!|<span style="color:#00a968">以下の投稿記事のロックを解除することが出来ます</span>|g;
	} 
	if ($lock == 0)  {
		$tmpl =~ s|!lock_com!|<span style="color:#dc143c">以下の投稿記事をロックして閉じることが出来ます</span>|g;
		$tmpl =~ s|!open_com!||g;
	}


	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  投稿フォーム
#-----------------------------------------------------------
sub form_page {
	passwd2(%in) if ($cf{usr_password} ne '');

	# クッキー取得
	my @cook = ref_get_cookie();
	$cook[2] ||= 'http://';
	my $ref_no = $cook[3];

	# 追加添付ファイル数
	my ($op_attach,$attach,$extxt,$att_msg,$gfc,$refno);

	$ref_no = $ref_no ne '' ? $ref_no : $cf{attach_defo};

	if ($ref_no > 0) {
		foreach $gfc (1 .. $ref_no) {
			$gfc++;
			$attach .= qq|<div id="attachment">添付 $gfc : <label><input type="file" name="upfile$gfc" id="upload$gfc" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;<input type="text" name="imgtext$gfc" class="reg-imgtxt" placeholder="添付 $gfc タイトル" value="" size="40"></div>\n|;
		}
	}

	if ($cf{add_usermax} > 0){
		foreach (@{$cf{addattach}}) {

			$refno = $_;
			$refno = "　　" . $refno;
			$refno =~ s/0/０/g;
			$refno =~ s/1/１/g;
			$refno =~ s/2/２/g;
			$refno =~ s/3/３/g;
			$refno =~ s/4/４/g;
			$refno =~ s/5/５/g;

			if ($ref_no == $_) {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			} else {
				$op_attach .= qq|<option value="$_">$refno個</option>\n|;
			}
		}
	}

		if ($ref_no == 0) { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームを追加出来ます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		} elsif ($ref_no == 5) { 
			$att_msg .= qq|<small><span style="color:#d83473">参照フォームは５個が限度です</span> <img src="./cmn/arrow-gfno01.png" alt="追加" class="icon2"></small>\n|;
		} else { 
			$att_msg .= qq|<small><span style="color:#00885a">参照フォームの数を変更できます</span> <img src="./cmn/arrow-gfno.png" alt="追加" class="icon2"></small>\n|;
		}

	# 画像認証作成
	my ($str_plain,$str_crypt,$regikey,$JavaScript);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt, $regikey, $JavaScript) = cap::make( $cf{captcha_key}, $cf{cap_len} );
		$JavaScript=&mkJavaScript($regikey,'myFORM') if($cf{cap_len} == 3);
	}

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $color;
	foreach (0 .. $#col) {
		if ($_ == $cook[3]) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# ヘッドラインプルダウン
        my @col3 = split(/\s+/, $cf{colors_b});
        my @col4 = split(/\s+/, $cf{colors_b2});
	my $color_b;
	if ($cook[4] eq "") { $cook[4] = $col3[0]; }
	foreach (0 .. $#col3) {
		if ($cook[4] eq $col3[$_]) {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]" selected>$col4[$_]</option>\n|;
		} else {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]">$col4[$_]</option>\n|;
		}
	}

	open(IN,"$cf{tmpldir}/form.html") or &error("open err: form.html");
	print "Content-type: text/html; charset=utf-8\n\n";
	while(<IN>) {

	# 文字置換
		s/!str_crypt!/$str_crypt/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!homepage!/$cf{homepage}/g;
		s/!reg-nam!/$cook[0]/g;
		s/!reg-eml!/$cook[1]/g;
		s/!reg-url!/$cook[2]/g;
		s/!color!/$color/g;
		s/<!-- op_color2 -->/$color_b/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s|!bbs_css!|$cf{cmnurl}/bbs.css|g;
		s/<!-- attach -->/$attach/g;
		s/<!-- add_attach -->/$op_attach/g;
		s/!att_msg!/$att_msg/g;

		$ref_no =~ s/0/０/g;
		$ref_no =~ s/1/１/g;
		$ref_no =~ s/2/２/g;
		$ref_no =~ s/3/３/g;
		$ref_no =~ s/4/４/g;
		$ref_no =~ s/5/５/g;

		s/!ref_no!/$ref_no/g;
		print;
	}
	close(IN);
	# フッタ
	footer();
	exit;
}

#-----------------------------------------------------------
#  パスワードフォーム
#-----------------------------------------------------------
sub p_form {

	$in{no} =~ s/\D//g;

	# クッキー取得
	my @cook = get_cookie();
	$cook[2] ||= 'http://';

	# 記事
	my ($name,$pwd);
	open(DAT,"+< $cf{datadir}/log.cgi") || &error("Open Error: $cf{datadir}/log.cgi");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);

		if ($in{no} == $no) {
			$name = $name;
			$sub = $sub;
			$pwd = $pw;
			last;
		}
	}
	close(DAT);

	# パスワード未設定
	error("この記事はパスワード未設定のため編集できません") if ($pwd eq '');

		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		$com = auto_link($com) if ($cf{autolink});
		if (length($com) > 135) { $com = substr($com,0,140); $com .= '.....(略)'; }

		# 添付
		if (defined($ex1)) {
			$com = att_file_pincode($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1);
		}

		my $clip;
			$clip = attach_pincode($no,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$name,$sub,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

	my @col = split(/\s+/, $cf{colors});

	open(IN,"$cf{tmpldir}/p_form.html") or &error("open err: p_form.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!url!/$url/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s/!cmnurl!/$cf{cmnurl}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/bbs.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!pwd!/$pw/g;
	$tmpl =~ s/!color2!/$col2/g;
	$tmpl =~ s/<!-- clip -->/$clip/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  YouTube貼付方法
#-----------------------------------------------------------
sub expound_page {

	open(IN,"$cf{tmpldir}/expound.html  ") or error("open err: expound.html  ");
	my $tmpl = join('', <IN>);
	close(IN);

	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/clip.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  ワード検索
#-----------------------------------------------------------
sub find_data {
	# 引数取得
	$in{word} =~ s|<br>||g;
	$in{cond} =~ s/\D//g;
	$in{pg} =~ s/\D//g;
	$in{pg} ||= 0;
	
	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $op_cond;
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op_cond .= qq|<option value="$_" selected>$op{$_}</option>\n|;
		} else {
			$op_cond .= qq|<option value="$_">$op{$_}</option>\n|;
		}
	}
	
	# 検索実行
	my ($i,@log) = search($in{word},$in{cond}) if ($in{word} ne '');
	
	# 検索用
	my $ewd = $in{word};
	$ewd =~ s/(\W)/"%".unpack("H2",$1)/ego;
	$ewd =~ tr/ /+/;
	my $param = "&amp;mode=find&amp;word=$ewd&amp;cond=$in{cond}";
	
	# ページ繰越ボタン作成
	my $pgbtn = make_btn($i,$in{pg},$param);
	
	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/find.html") or error("open err: find.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	# 文字置換
	$tmpl =~ s/!(bbs_cgi|bbs_title|cmnurl)!/$cf{$1}/g;
	$tmpl =~ s/<!-- op_cond -->/$op_cond/;
	$tmpl =~ s/!word!/$in{word}/;
	$tmpl =~ s/!page_btn!/$pgbtn/;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="$1" class="icon">|g;

	if ($in{word} eq '') {
		$tmpl =~ s|<!-- find_result -->.+?<!-- /find_result -->||s;
	} else {
		$tmpl =~ s/!result!/$i/g;
	}
	
	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ m|(.+)<!-- loop -->(.+?)<!-- /loop -->(.+)|s
			? ($1,$2,$3)
			: error("テンプレート不正");
	
	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;
	
	foreach (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{iconurl}/house04.png" class="icon" alt="home"></a>|;
		$com = auto_link($com) if ($cf{autolink});
		
		my @col = split(/\s+/, $cf{colors});

		# 添付
		if (defined($ex1)) {
			$com = att_file($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1);
		}

		my $clip;
			$clip = attach($no,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$name,$sub,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!url!/$url/g;
		$tmp =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
		$tmp =~ s/!col2!/$col2/g;
		$tmp =~ s/<!-- clip -->/$clip/;
		$tmp =~ s|!clip!|<img src="$cf{cmnurl}/paper_clip.png" border="0" alt="" class="img_a">|g;

		if ($lock == 0 || $lock == '') {
			$tmp =~ s|<!-- lock -->(.+?)<!-- /lock -->||s;
		}

		print $tmp;
	}
	
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  検索実行
#-----------------------------------------------------------
sub search {
	my ($word,$cond) = @_;
	
	# キーワードを配列化
	$word =~ s/\x81\x40/ /g;
	my @wd = split(/\s+/,$word);
	
	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';
	
	# 検索処理
	my ($i,@log);
	open(IN,"$cf{datadir}/log.cgi") or error("open err: log.cgi");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/);
		
		my $flg;
		foreach my $wd (@wd) {
			if ("$name $eml $sub $com $url" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);
		
		$i++;
		next if ($i < $in{pg} + 1);
		next if ($i > $in{pg} + $cf{pg_max});
		
		push(@log,$_);
	}
	close(IN);
	
	# 検索結果
	return ($i,@log);
}

#-----------------------------------------------------------
#  コメント返信フォーム
#-----------------------------------------------------------
sub rescom_form {

	# レスカウント
	my (@log,%res_cnt,@reply,$max_res_flg);
	open(IN,"$cf{rescnt}");
	while(<IN>) {
		my ($no,$date,$name,$res) = split(/<>/);

		$res_cnt{$no} = $res;

	}

	# 表示データ認識
	my (@log,@reply,%res_cnt,%res,$max_res_flg);
	open(IN,"$cf{datadir}/log.cgi") or &error("open err: $cf{datadir}/log.cgi");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/);

		if ($in{no} == $no) {

			$res{sub} = $sub;

		push(@log,$_);
		}
	}

	# コメント返信ログ
	open(IN,"$cf{reslog}");
	while(<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		# 該当レス
		if ($in{no} == $ent) {

			push(@reply,$_);

			$res_cnt{$ent}++;
		}
	}
	close(IN);

	# クッキー取得
	my ($c_nam,$c_url) = get_cookie();
	$c_url ||= 'http://';

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/res_read.html") or error("open err: res_read.html");
	my $tmpl = join('', <IN>);
	close(IN);

	open(IN,"$cf{tmpldir}/res.html") or error("open err: res.html");
	my $resloop = join('', <IN>);
	close(IN);

	# 画像認証作成
	my ($str_plain,$str_crypt);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain,$str_crypt) = cap::make($cf{captcha_key},$cf{cap_len});
		$tmpl =~ s/!str_crypt!/$str_crypt/g;
	} else {
		$tmpl =~ s/<!-- captcha_begin -->.+<!-- captcha_end -->//s;
	}

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl,$resloop);

	# 文字置換
	for ($head,$foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
		s|!form_css!|$cf{cmnurl}/form.css|g;
		s|!input_file_btn_css!|$cf{cmnurl}/input_file_btn.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!c_name!/$c_nam/g;
		s/!c_url!/$c_url/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!max_res!/$cf{max_res}/g;
		s/!no!/$in{no}/g;
		s/!r-sub!/$res{sub}/g;
		s/!ptop-icon!/<img src="$cf{ico_ptop}" alt="pagetop">/g;

	# 返信フォームのサブミットボタン(ディスエイブル)｛エラーメッセージを表示したい場合は、コメントアウト｝
	#if ($res_cnt{$in{no}} >= $cf{max_res}) {
	#	s|<input class="submit-button" type="submit" value="Submit" />|<input class="submit-button" type="submit" value="Submit" disabled />|g;
	#}

	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach my $log (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/, $log);
		$name = qq|<a href="mailto:$eml">$name</a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon" alt="HOME" /></a>|;
		$com = auto_link($com) if ($cf{autolink});
		$eml = qq|<a href="mailto:$eml"><img src="$cf{ico_mail}" class="icon" alt="" /></a>| if ($eml);

		if (!$res_cnt{$no}) { $res_cnt{$no} = 0; }

		# YouTube
		if ($cf{tube}) {
			$com =~ s|!([\w\-]{1,})!|<span class="imageCover"><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="$cf{cmnurl}/youtube.png" width="64" height="64" class="coverImage"></a></span>|g;
		}

		# 添付
		if (defined($ex1)) {
			$com = att_file($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1);
		}

		my $clip;
			$clip = attach($no,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$name,$sub,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

		my @col = split(/\s+/, $cf{colors});

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!email!/$eml/g;
		$tmp =~ s/!home!/$url/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!color2!/$col2/g;
		$tmp =~ s|!ico_edit!|<a href="$cf{bbs_cgi}?mode=p_form&no=$no"><img src="$cf{iconurl}/Edit02.png" alt="" /></a>|g;
		$tmp =~ s/!res_com!/$cf{bbs_cgi}?mode=rescom&no=$no/g;
		$tmp =~ s/!ico_com!/$cf{ico_com}/g;
		$tmp =~ s|!ico_com02!|<img src="$cf{iconurl}/discuss02.png" alt="" class="icon">|g;
		$tmp =~ s/!res_cnt!/$res_cnt{$no}/g;

	# 返信コメント
	if ($res_cnt{$no} == 0) {
		$tmp =~ s|<!-- rescomment -->|<span style="color:#ed6d46">《この記事にはまだコメントがありません》</span>|g;
	} else {
		$tmp =~ s/<!-- rescomment -->//g;
	}

		print $tmp;

	foreach (@reply) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);
		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }

		my $clip;
		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{r_upldir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{r_upldir}/$no-s$ex") {
					$op = qq|src="$cf{r_uplurl}/$no-s$ex"|;
				} else {
					($w,$h) = resize_res($w,$h);
					$op = qq|src="$cf{r_uplurl}/$no$ex" width="$w" height="$h"|;
				}
				$clip .= qq|<div id="clip"><a href="$cf{r_uplurl}/$no$ex" class="fancybox-effects-c" title=""><img $op border="0" class="snapshot"></a></div>|;

			# 指定アイコン (画像以外)
			} elsif (defined($cf{icons}{$ex})) {
				my $size = get_size("$cf{r_upldir}/$no$ex");
				$com .= qq|<div class="clip">添付: <a href="$cf{r_uplurl}/$no$ex" target="_blank"><img src="$cf{icourl}/$cf{icons}{$ex}" alt="" class="icon" /></a> (<b style="color:#00a1e9">$size</b>)</div>|;
			}
		}

			my $tmp = $resloop;
			$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
			$tmp =~ s|!com_edit!|<a href="$cf{bbs_cgi}?mode=compwd&no=$no"><img src="$cf{iconurl}/edit03.png" alt="" class="icon" /></a>|g;
			$tmp =~ s/!r_name!/$name/g;
			$tmp =~ s/!r_date!/$date/g;
			$tmp =~ s/!r_url!/$url/g;
			$tmp =~ s/!r_comment!/$com/g;
			$tmp =~ s/<!-- ext -->/$clip/g;
			print $tmp;
		}
	}
	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメントワード検索
#-----------------------------------------------------------
sub res_find_data {
	# 引数取得
	$in{word} =~ s/[\r\n]//g;
	$in{cond} =~ s/\D//g;
	$in{pg} =~ s/\D//g;
	$in{pg} ||= 0;

	# 検索条件プルダウン
	my %op = (1 => 'AND', 0 => 'OR');
	my $op_cond;
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op_cond .= qq|<option value="$_" selected>$op{$_}\n|;
		} else {
			$op_cond .= qq|<option value="$_">$op{$_}\n|;
		}
	}

	# 検索実行
	my ($i,@reply) = res_search($in{word},$in{cond}) if ($in{word} ne '');
	
	# 検索用
	my $ewd = $in{word};
	$ewd =~ s/(\W)/"%".unpack("H2",$1)/ego;
	$ewd =~ tr/ /+/;
	my $param = "&amp;mode=res_find&amp;word=$ewd&amp;cond=$in{cond}";

	# ページ繰越ボタン作成
	my $pgbtn = make_resbtn($i,$in{pg},$param);

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/res_find.html") or error("open err: res_find.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot) = $tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s
			? ($1,$2,$3) : error("テンプレート不正");

	# 文字置換
	for ($head,$foot) {
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
		s/!bbs_cgi!/$cf{bbs_cgi}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/<!-- op_cond -->/$op_cond/;
		s/!word!/$in{word}/;
		s/!page_btn!/$pgbtn/;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@reply) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);
		if ($url) { $url = qq |&lt;<a href="$url" target="_blank">URL</a>&gt;|; }

		my $clip;
		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{r_upldir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち (<b style="color:#FF9966">$size</b>)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{r_upldir}/$no-s$ex") {
					$op = qq|src="$cf{r_uplurl}/$no-s$ex"|;
				} else {
					($w,$h) = resize_res($w,$h);
					$op = qq|src="$cf{r_uplurl}/$no$ex" width="$w" height="$h"|;
				}
				$clip .= qq|<div id="clip"><a href="$cf{r_uplurl}/$no$ex" class="fancybox-effects-c" title=""><img $op border="0" class="snapshot"></a></div>|;

			# 指定アイコン (画像以外)
			} elsif (defined($cf{icons}{$ex})) {
				my $size = get_size("$cf{r_upldir}/$no$ex");
				$com .= qq|<div class="clip">添付: <a href="$cf{r_uplurl}/$no$ex" target="_blank"><img src="$cf{icourl}/$cf{icons}{$ex}" alt="" class="icon" /></a> (<b style="color:#00a1e9">$size</b>)</div>|;
			}
		}

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="" class="icon" />|g;
		$tmp =~ s|!com_edit!|<a href="$cf{bbs_cgi}?mode=compwd&no=$no"><img src="$cf{iconurl}/edit03.png" alt="" class="icon" /></a>|g;
		$tmp =~ s/!r_num!/$no/g;
		$tmp =~ s/!r_name!/$name/g;
		$tmp =~ s/!r_date!/$date/g;
		$tmp =~ s/!r_url!/$url/g;
		$tmp =~ s/!r_comment!/$com/g;
		$tmp =~ s/<!-- ext -->/$clip/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  コメント検索実行
#-----------------------------------------------------------
sub res_search {
	my ($word,$cond) = @_;

	# キーワードを配列化
	$word =~ s/\x81\x40/ /g;
	my @wd = split(/\s+/,$word);

	# UTF-8定義
	my $byte1 = '[\x00-\x7f]';
	my $byte2 = '[\xC0-\xDF][\x80-\xBF]';
	my $byte3 = '[\xE0-\xEF][\x80-\xBF]{2}';
	my $byte4 = '[\xF0-\xF7][\x80-\xBF]{3}';

	# 検索処理
	my ($i,@reply);
	open(IN,"$cf{reslog}") or error("open err: $cf{datadir}/log.cgi");
	while (<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

		my $flg;
		foreach my $wd (@wd) {
			if ("$date $name $url $com" =~ /^(?:$byte1|$byte2|$byte3|$byte4)*?\Q$wd\E/i) {
				$flg++;
				if ($cond == 0) { last; }
			} else {
				if ($cond == 1) { $flg = 0; last; }
			}
		}
		next if (!$flg);

		$i++;
		next if ($i < $in{pg} + 1);
		next if ($i > $in{pg} + $cf{max_resfind});
		
		push(@reply,$_);
	}
	close(IN);

	# 検索結果
	return ($i,@reply);
}

#-----------------------------------------------------------
#  コメント返信パスワードフォーム
#-----------------------------------------------------------
sub com_pwdform {

	$in{no} =~ s/\D//g;

	# クッキー取得
	my @cook = get_cookie();
	$cook[2] ||= 'http://';

	# 記事
	my ($name,$pwd);
	open(DAT,"+< $cf{reslog}") || &error("Open Error: $cf{reslog}");
	eval "flock(DAT, 2);";
	while (<DAT>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);

	if ($in{no} == $no) {
		$name = $name;
		$sub = $sub;
		$pwd = $pw;
		last;
		}
	}
	close(DAT);

	# パスワード未設定
	&error("このコメントはパスワード未設定のため編集できません") if ($pwd eq '');

	my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		$com = auto_link($com) if ($cf{autolink});
		if (length($com) > 110) { $com = substr($com,0,120); $com .= '.....(略)'; }

		# 添付あり
		if ($ex) {
			# 確認待ち
			if ($cf{imgcheck} && $chk != 1) {
				my $size = get_size("$cf{r_upldir}/$no$ex");
				$com .= qq|<p>[添付]: 認証待ち ($size)</p>|;

			# 添付画像
			} elsif ($ex =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{r_upldir}/$no-s$ex") {
					$op = qq|src="$cf{r_uplurl}/$no-s$ex"|;
				} else {
					($w,$h) = resize_pwd($w,$h);
					$op = qq|src="$cf{r_uplurl}/$no$ex" width="$w" height="$h"|;
				}
		# 画像はコメントの下
		if ($cf{imgpoint} == 1) {
			$com .= qq|<p><a href="$cf{r_uplurl}/$no$ex" class="fancybox-effects-c"><img $op border="0"></a></p>|;
		# 画像はコメントの左（廻り込み）
		} else {
			$com = qq|<a href="$cf{r_uplurl}/$no$ex" class="fancybox-effects-c"><img $op border="0" align="left" hspace="12"></a>$com<br clear="all">|;
		}

		# 指定アイコン (画像以外)
		} elsif (defined($cf{icons}{$ex})) {
			my $size = get_size("$cf{r_upldir}/$no$ex");
			$com .= qq|<div class="clip">添付: <a href="$cf{r_uplurl}/$no$ex" target="_blank"><img src="$cf{icourl}/$cf{icons}{$ex}" alt="" class="icon" /></a> (<b style="color:#00a1e9">$size</b>)</div>|;
		}
		}

	my @col = split(/\s+/, $cf{colors});

	open(IN,"$cf{tmpldir}/res_pwdform.html") or &error("open err: res_pwdform.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# 文字置換
	$tmpl =~ s/!sub!/$sub/g;
	$tmpl =~ s/!name!/$name/g;
	$tmpl =~ s/!no!/$in{no}/g;
	$tmpl =~ s|!comment!|$com|g;
	$tmpl =~ s/!date!/$date/g;
	$tmpl =~ s/!url!/$url/g;
	$tmpl =~ s/!([a-z]+_cgi)!/$cf{$1}/g;
	$tmpl =~ s/!bbs_title!/$cf{bbs_title}/g;
	$tmpl =~ s|!bbs_css!|$cf{cmnurl}/clip.css|g;
	$tmpl =~ s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
	$tmpl =~ s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
	$tmpl =~ s/!num!/$no/g;
	$tmpl =~ s/!pwd!/$pwd/g;

	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  コメント返信アルバム機能
#-----------------------------------------------------------
sub res_album {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{r_max_img_w};
	$cf{max_img_h} = $cf{r_max_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/res_album.html") or error("open err: res_album.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# データ読み込み
	my ($i,@img);
	open(IN,"$cf{reslog}") or error("open err: $cf{reslog}");
	while (<IN>) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/);
		next if ($cf{imgcheck} && $chk eq '0');
		next if ($ex !~ /(jpg|png|gif)$/);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{max_resalbum});

		# 画像データ収集
		push(@img,"$no\t$ent\t$ex\t$w\t$h");
	}
	close(IN);

	# 繰越ボタン
	my $page_btn = make_pgbtn($i,$pg,'&mode=res_album',$cf{max_resalbum});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s/!page_btn!/$page_btn/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($no,$ent,$ex,$w,$h) = split(/\t/);

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!ent!/$ent/g;
		$tmp =~ s/!res-image!/res_image($no,$ent,$ex,$w,$h)/eg;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  見出しサンプル
#-----------------------------------------------------------
sub headline_page {
	# 時間取得
	my ($date,$time) = get_time2();

	# テンプレート認識
	open(IN,"$cf{tmpldir}/headline.html") or &error("open err: headline.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot);
	if ($tmpl =~ /(.+)<!-- loop_begin -->(.+)<!-- loop_end -->(.+)/s) {
		($head,$loop,$foot) = ($1, $2, $3);
	} else {
		&error("テンプレートが不正です");
	}

	# 文字置換
	foreach ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!version!/$cf{version}/g;
		s|!bbs_css!|$cf{cmnurl}/bbs.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!bbs_title!/$cf{bbs_title}/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

        my @col3 = split(/\s+/, $cf{colors_b});
        my @col4 = split(/\s+/, $cf{colors_b2});
	foreach (0 .. $#col3) {

		my $tmp = $loop;
		$tmp =~ s/!name!/$col4[$_]/g;
		$tmp =~ s|!color_b!|<div class="article_samp"><h5 style="background:$col3[$_]">$date</h5>
					<div class="art-com_samp"><b style="color:$col3[$_]; font-size:10pt;">$col4[$_]</b></div>
					</div>|g;
		print $tmp;
	}

	# フッタ
	footer($foot);
	exit;
}

#-----------------------------------------------------------
#  枠サンプル用時間取得
#-----------------------------------------------------------
sub get_time2 {
	$ENV{TZ} = "JST-9";
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 日時のフォーマット
	my $date = sprintf("%04d/%02d/%02d(%s)",
			$year+1900,$mon+1,$mday,$week[$wday],$hour,$min);

	return ($date, $time);
}

#-----------------------------------------------------------
#  留意事項
#-----------------------------------------------------------
sub note_page {
	# 許可拡張子（表示用）
	my $ftype = ext_file();
	
	open(IN,"$cf{tmpldir}/note.html") or error("open err: note.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!filetype!/$ftype/g;
	$tmpl =~ s/!([a-z]+_cgi|homepage|cmnurl|bbs_title)!/$cf{$1}/g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  アルバム機能
#-----------------------------------------------------------
sub view_page {
	# ページ数定義
	my $pg = $in{pg} || 0;

	# 画像サイズ再定義
	$cf{max_img_w} = $cf{alb_img_w};
	$cf{max_img_h} = $cf{alb_img_h};

	# テンプレート認識
	open(IN,"$cf{tmpldir}/view.html") or error("open err: view.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	# データ読み込み
	my ($i,@img);
	open(IN,"$cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$color2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/);
		next if ($cf{imgcheck} && $chk eq '0');
		next if ($ex1 !~ /(\.jpe?g|\.png|\.gif)$/i);

		$i++;
		next if ($i < $pg + 1);
		next if ($i > $pg + $cf{max_albums});

		# 画像データ収集
		push(@img,"$no\t$sub\t$name\t$ex1\t$w1\t$h1\t$ex2\t$w2\t$h2\t$ex3\t$w3\t$h3\t$ex4\t$w4\t$h4\t$ex5\t$w5\t$h5\t$ex6\t$w6\t$h6\t$imgtxt1\t$imgtxt2\t$imgtxt3\t$imgtxt4\t$imgtxt5\t$imgtxt6");
	}
	close(IN);

	# 繰越ボタン
	my $page_btn = make_pgbtn($i,$pg,'&mode=view',$cf{max_albums});

	# 文字置換
	for ($head, $foot) {
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!page_btn!/$page_btn/g;
	}

	# 画面展開
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@img) {
		my ($no,$sub,$name,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/\t/);

		my $clip;
			$clip = image($no,$sub,$name,$ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!image!/$clip/g;
		my $capt = "投稿者 : $name　タイトル : $sub";
		$tmp =~ s/!caption!/$capt/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
}

#-----------------------------------------------------------
#  繰越ボタン作成 (アルバム)
#-----------------------------------------------------------
sub make_pgbtn {
	my ($i,$pg,$stat,$max) = @_;
	$max ||= $cf{max_resalbum};

	# ページ繰越定義
	my $next = $pg + $max;
	my $back = $pg - $max;

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>!);
			} else {
				push(@pg,qq!<li><a href="$cf{bbs_cgi}?pg=$y$stat">$x</a></li>!);
			}
			$x++;
			$y += $max;
			$z -= $max;

			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$cf{bbs_cgi}?pg=$back$stat">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$cf{bbs_cgi}?pg=$next$stat">&raquo;</a></li>\n!;
	}
	# 結果を返す
	return $ret ? qq|<ul class="pager">$ret</ul>| : '';
}

#-----------------------------------------------------------
#  過去ログページ
#-----------------------------------------------------------
sub past_page {
	# 過去ログ番号
	open(IN,"$cf{nofile}") or error("open err: $cf{nofile}");
	my $pastnum = <IN>;
	close(IN);

	my $pastnum = sprintf("%04d",$pastnum);
	$in{pno} =~ s/\D//g;
	$in{pno} ||= $pastnum;

	# プルダウンタグ作成
	my $op_pno;
	for ( my $i = $pastnum; $i > 0; $i-- ) {
		$i = sprintf("%04d",$i);

		if ($in{pno} == $i) {
			$op_pno .= qq|<option value="$i" selected="selected">$i</option>\n|;
		} else {
			$op_pno .= qq|<option value="$i">$i</option>\n|;
		}
	}

	# ページ数
	my $pg = $in{pg} || 0;

	# 初期化
	my ($hit,$page_btn,@log);

	# 対象ログ定義
	my $file = "$cf{pastdir}/" . sprintf("%04d", $in{pno}) . ".cgi";

	# ワード検索
	if ($in{find} && $in{word} ne '') {

		# 検索
		($hit,@log) = search($file,$in{list},'past');

		# 結果
		$page_btn = "検索結果：<b>$hit</b>件 &nbsp;&nbsp;" . pgbtn_old($hit,$in{pno},$pg,$in{list},'find');

	# ログ一覧
	} else {

		# 過去ログオープン
		my $i = 0;
		open(IN,"$file") or error("open err: $file");
		while(<IN>) {
			$i++;
			next if ($i < $pg + 1);
			next if ($i > $pg + $cf{pg_maxp} );

			push(@log,$_);
		}
		close(IN);

		# 繰越ボタン作成
		$page_btn = pgbtn_old($i,$in{pno},$pg);
	}

	# プルダウン作成（検索条件）
	my %op = make_op();

	# テンプレート読み込み
	open(IN,"$cf{tmpldir}/past.html") or error("open err: past.html");
	my $tmpl = join('', <IN>);
	close(IN);

	# テンプレート分割
	my ($head,$loop,$foot) = split(/<!-- loop -->/,$tmpl);

	if ($in{change}) { $in{word} = ''; }

	my @col = split(/\s+/,$cf{colors});

	# 文字置換
	for ($head, $foot) {
		s/!past_num!/$in{pno}/g;
		s/!bbs_url!//g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s|!bbs_css!|$cf{cmnurl}/clip.css|g;
		s|!mob_css!|$cf{cmnurl}/clip-mobile.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/<!-- op_pno -->/$op_pno/g;
		s/<!-- op_(\w+) -->/$op{$1}/g;
		s/!word!/$in{word}/g;
		s/!page_btn!/$page_btn/g;
	}

	# 画面表示
	print "Content-type: text/html; charset=utf-8\n\n";
	print $head;

	foreach (@log) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$color2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/);
		$eml = qq|<a href="mailto:$eml"><img src="$cf{ico_mail}" class="icon"></a>| if ($eml);
		$url &&= qq|<a href="$url" target="_blank"><img src="$cf{ico_home}" class="icon"></a>|;
		$com = auto_link($com) if ($cf{autolink});

		# YouTube
		if ($cf{tube}) {
			$com =~ s|!([\w\-]{1,})!|<span class="imageCover"><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="https://i.ytimg.com/vi/$1/mqdefault.jpg" width="240" height="154" class="baseImage"></a><a class="fancybox-media" href="http://www.youtube.com/watch?v=$1" title="Youtube"><img src="$cf{cmnurl}/youtube.png" width="64" height="64" class="coverImage"></a></span>|g;		}

		# 添付
		if (defined($ex1)) {
			$com = att_file($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1);
		}

		my $clip;
			$clip = attach($no,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$name,$sub,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6);
			$clip &&= "<p>$clip</p>";

		# 記事書き出し
		my $tmp = $loop;
		$tmp =~ s/!num!/$no/g;
		$tmp =~ s/!sub!/$sub/g;
		$tmp =~ s/!name!/$name/g;
		$tmp =~ s/!email!/$eml/g;
		$tmp =~ s/!home!/$url/g;
		$tmp =~ s/!date!/$date/g;
		$tmp =~ s|!comment!|<span style="color:$col[$col]">$com</span>|g;
		$tmp =~ s/<!-- clip -->/$clip/g;
		$tmp =~ s/!color2!/$color2/g;
		print $tmp;
	}

	# フッタ
	footer($foot);
	exit;
}

#-----------------------------------------------------------
#  プルダウン作成 [ 検索条件 ]
#-----------------------------------------------------------
sub make_op {
	my %op;
	my %cond = (1 => 'AND', 0 => 'OR');
	foreach (1,0) {
		if ($in{cond} eq $_) {
			$op{cond} .= qq|<option value="$_" selected>$cond{$_}\n|;
		} else {
			$op{cond} .= qq|<option value="$_">$cond{$_}\n|;
		}
	}
	for ( my $i = 10; $i <= 30; $i += 5 ) {
		if ($in{list} == $i) {
			$op{list} .= qq|<option value="$i" selected>$i件\n|;
		} else {
			$op{list} .= qq|<option value="$i">$i件\n|;
		}
	}
	return %op;
}

#-----------------------------------------------------------
#  繰越ボタン作成 [ 過去ログ ]
#-----------------------------------------------------------
sub pgbtn_old {
	my ($i,$pno,$pg,$stat) = @_;

	# ページ繰越定義
	$cf{pg_maxp} ||= 10;
	my $next = $pg + $cf{pg_maxp} ;
	my $back = $pg - $cf{pg_maxp} ;

	my $link;
	if ($stat eq 'past') {
		my $wd = url_enc($in{word});
		$link = "$cf{bbs_cgi}?mode=$in{mode}&amp;pno=$pno&amp;find=1&amp;word=$wd";
	} else {
		$link = "$cf{bbs_cgi}?mode=$in{mode}&amp;pno=$pno";
	}

	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<li><span>$x</span></li>\n!);
			} else {
				push(@pg,qq!<li><a href="$link&amp;pg=$y">$x</a></li>\n!);
			}
			$x++;
			$y += $cf{pg_maxp} ;
			$z -= $cf{pg_maxp} ;
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('', @pg);
	if ($back >= 0) {
		$ret = qq!<li><a href="$link&amp;pg=$back">&laquo;</a></li>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<li><a href="$link&amp;pg=$next">&raquo;</a></li>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<ul class="pager">\n$ret</ul>| : '';
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<div id="footer">
	- <a href="http://www.kent-web.com/" target="_top">$cf{version}</a> -<br>
	++ <a href="http://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</div>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  自動リンク
#-----------------------------------------------------------
sub auto_link {
	my $text = shift;
	
	$text =~ s/(s?https?:\/\/([\w-.!~*'();\/?:\@=+\$,%#]|&amp;)+)/<a href="$1" target="_blank">$1<\/a>/g;
	return $text;
}

#-----------------------------------------------------------
#  参照フォームクッキー取得
#-----------------------------------------------------------
sub ref_get_cookie {
	my $ref_no = @_;

	# クッキー取得
	$ENV{HTTP_COOKIE} =~ /$cf{cookie_id2}=(.+);?/;
	my $cookie = $1;
	
	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cookie) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}

	if ($ref_no == 0) { $ref_no = "0"; }

	return @cook;
}

#-----------------------------------------------------------
#  クッキー取得
#-----------------------------------------------------------
sub get_cookie {
	# クッキー取得
	$ENV{HTTP_COOKIE} =~ /$cf{cookie_id}=(.+);?/;
	my $cookie = $1;
	
	# URLデコード
	my @cook;
	foreach ( split(/<>/,$cookie) ) {
		s/%([0-9A-Fa-f][0-9A-Fa-f])/pack("H2",$1)/eg;
		s/[&"'<>]//g;
		
		push(@cook,$_);
	}
	return @cook;
}

#-----------------------------------------------------------
#  添付リンク
#-----------------------------------------------------------
sub att_file {
	my ($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1) = @_;
	
	# 未公開
		if ($ex1) {
			# 確認待ち
			if ($cf{imgcheck}) {
				my $size = get_size("$cf{imgdir}/$no-1$ex1");
				$com .= qq|<p>[添付]: 認証待ち ($size)</p>|;

			# 添付画像
			} elsif ($ex1 =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{imgdir}/$no-s-1$ex1") {
					$op = qq|src="$cf{imgurl}/$no-s-1$ex1"|;
				} else {
					($w1,$h1) = resize($w1,$h1);
					$op = qq|src="$cf{imgurl}/$no-1$ex1" width="$w1" height="$h1"|;
				}

			# 画像はコメントの下
			if ($cf{imgpoint} == 1) {
				$com .= qq|<p><a href="$cf{imgurl}/$no-1$ex1" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub )&nbsp;ファイル名：$imgtxt1"><div class="snap"><img $op class="snapshot"></a><br />$imgtxt1</div></p>|;
			# 画像はコメントの左（廻り込み）
			} else {
				$com = qq|<a href="$cf{imgurl}/$no-1$ex1" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub )&nbsp;ファイル名：$imgtxt1"><div class="snap"><img $op class="snapshot"></a><br />$imgtxt1</div> $com<br clear="all">|;
			}

			# 指定アイコン (画像以外)
			} elsif (defined($cf{icons}{$ex1})) {
				my $size = get_size("$cf{imgdir}/$no-1$ex1");
				$com .= qq|<p><a href="$cf{imgurl}/$no-1$ex1" target="_blank"><div class="attachment"><img src="$cf{icourl}/$cf{icons}{$ex1}" alt="" class="icon" /></a> (<b style="color:#00a1e9">$size</b>) <br />$imgtxt1</div> </p>|;
			}
		}
	
	return $com;
}

#-----------------------------------------------------------
#  追加添付表示
#-----------------------------------------------------------
sub attach {
	my ($com) = @_;
	my $no = shift;
	my (%ex,%w,%h,$name,$sub,%imgtxt);
	($ex{2},$w{2},$h{2},$ex{3},$w{3},$h{3},$ex{4},$w{4},$h{4},$ex{5},$w{5},$h{5},$ex{6},$w{6},$h{6},$name,$sub,$imgtxt{2},$imgtxt{3},$imgtxt{4},$imgtxt{5},$imgtxt{6}) = @_;

	my $ret;
	foreach my $i (2 .. 6) {
		next if (!$ex{$i});

		# 未公開
		if ($ex{$i} eq 'hide') {
		$com .= qq|<p>[添付]: 認証待ち</p>|;

		# 指定アイコン (画像以外)の場合
		#} elsif ($ex{$i} !~ /^\.(jpg|gif|png)$/) {
		} elsif (defined($cf{icons}{$ex{$i}})) {
			my $size = -s "$cf{imgdir}/$no-$i$ex{$i}";
			my $size = get_size("$cf{imgdir}/$no-$i$ex{$i}");
			$ret .= qq|<div  class="addclip"><a href="$cf{imgurl}/$no-$i$ex{$i}" target="_blank"><div class="attachment"><img src="$cf{icourl}/$cf{icons}{$ex{$i}}" alt="" /></a> (<b style="color:#00a1e9">$size</b>)<br />$imgtxt{$i}</div></div>|;

		# 画像の場合
		} elsif (-f "$cf{imgdir}/$no-s-$i$ex{$i}") {
			$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><div class="snap"><img src="$cf{imgurl}/$no-s-$i$ex{$i}" class="snapshot"></a><br />$imgtxt{$i}</div>|;

		} else {
			my ($w,$h) = resize($w{$i},$h{$i});
			$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><div class="snap"><img src="$cf{imgurl}/$no-$i$ex{$i}" width="$w" height="$h" class="snapshot"></a><br />$imgtxt{$i}</div>|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  画像表示(アルバム)
#-----------------------------------------------------------
sub image {
	my $no = shift;
	my ($sub,$name,%ex,%w,%h,%imgtxt);
	($sub,$name,$ex{1},$w{1},$h{1},$ex{2},$w{2},$h{2},$ex{3},$w{3},$h{3},$ex{4},$w{4},$h{4},$ex{5},$w{5},$h{5},$ex{6},$w{6},$h{6},$imgtxt{1},$imgtxt{2},$imgtxt{3},$imgtxt{4},$imgtxt{5},$imgtxt{6}) = @_;

	my $ret;
	foreach my $i (1 .. 6) {
		next if (!$ex{$i});
		if ($ex{$i} =~ /(\.jpe?g|\.png|\.gif)$/i) {
			if (-f "$cf{imgdir}/$no-s-$i$ex{$i}") {
				$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-buttons" data-fancybox-group="button" title="No.$no&nbsp;&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><img src="$cf{imgurl}/$no-s-$i$ex{$i}" class="alb_snapshot"></a>|;

			} else {
				my ($w,$h) = alb_resize($w{$i},$h{$i});
				$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-buttons" data-fancybox-group="button" title="No.$no&nbsp;&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><img src="$cf{imgurl}/$no-$i$ex{$i}" width="$w" height="$h" class="alb_snapshot"></a>|;
			}
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  パスワードフォーム・添付リンク
#-----------------------------------------------------------
sub att_file_pincode {
	my ($no,$com,$ex1,$w1,$h1,$name,$sub,$imgtxt1) = @_;
	
	# 未公開
		if ($ex1) {
			# 確認待ち
			if ($cf{imgcheck}) {
				my $size = get_size("$cf{imgdir}/$no-1$ex1");
				$com .= qq|<p>[添付]: 認証待ち ($size)</p>|;

			# 添付画像
			} elsif ($ex1 =~ /jpg$|gif$|png$/) {
				my $op;
				if (-f "$cf{imgdir}/$no-s-1$ex1") {
					$op = qq|src="$cf{imgurl}/$no-s-1$ex1"|;
				} else {
					($w1,$h1) = resize($w1,$h1);
					$op = qq|src="$cf{imgurl}/$no-1$ex1" width="$w1" height="$h1"|;
				}

			# 画像はコメントの下
			if ($cf{imgpoint} == 1) {
				$com .= qq|<p><a href="$cf{imgurl}/$no-1$ex1" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub )&nbsp;ファイル名：$imgtxt1"><div class="snap"><img $op class="snapshot" width="80px"></a><br />$imgtxt1</div></p>|;
			# 画像はコメントの左（廻り込み）
			} else {
				$com = qq|<a href="$cf{imgurl}/$no-1$ex1" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub )&nbsp;ファイル名：$imgtxt1"><div class="snap"><img $op class="snapshot" width="80px"></a><br />$imgtxt1</div> $com<br clear="all">|;
			}

			# 指定アイコン (画像以外)
			} elsif (defined($cf{icons}{$ex1})) {
				my $size = get_size("$cf{imgdir}/$no-1$ex1");
				$com .= qq|<p><a href="$cf{imgurl}/$no-1$ex1" target="_blank"><div class="attachment"><img src="$cf{icourl}/$cf{icons}{$ex1}" alt="" class="icon" /></a> (<b style="color:#00a1e9">$size</b>) <br />$imgtxt1</div> </p>|;
			}
		}
	
	return $com;
}

#-----------------------------------------------------------
#  パスワードフォーム・追加添付表示
#-----------------------------------------------------------
sub attach_pincode {
	my ($com) = @_;
	my $no = shift;
	my (%ex,%w,%h,$name,$sub,%imgtxt);
	($ex{2},$w{2},$h{2},$ex{3},$w{3},$h{3},$ex{4},$w{4},$h{4},$ex{5},$w{5},$h{5},$ex{6},$w{6},$h{6},$name,$sub,$imgtxt{2},$imgtxt{3},$imgtxt{4},$imgtxt{5},$imgtxt{6}) = @_;

	my $ret;
	foreach my $i (2 .. 6) {
		next if (!$ex{$i});

		# 未公開
		if ($ex{$i} eq 'hide') {
		$com .= qq|<p>[添付]: 認証待ち</p>|;

		# 指定アイコン (画像以外)の場合
		#} elsif ($ex{$i} !~ /^\.(jpg|gif|png)$/) {
		} elsif (defined($cf{icons}{$ex{$i}})) {
			my $size = -s "$cf{imgdir}/$no-$i$ex{$i}";
			my $size = get_size("$cf{imgdir}/$no-$i$ex{$i}");
			$ret .= qq|<div  class="addclip"><a href="$cf{imgurl}/$no-$i$ex{$i}" target="_blank"><div class="attachment"><img src="$cf{icourl}/$cf{icons}{$ex{$i}}" alt="" /></a> (<b style="color:#00a1e9">$size</b>)<br />$imgtxt{$i}</div></div>|;

		# 画像の場合
		} elsif (-f "$cf{imgdir}/$no-s-$i$ex{$i}") {
			$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><div class="snap"><img src="$cf{imgurl}/$no-s-$i$ex{$i}" width="80px" class="snapshot"></a><br />$imgtxt{$i}</div>|;

		} else {
			my ($w,$h) = resize($w{$i},$h{$i});
			$ret .= qq|<a href="$cf{imgurl}/$no-$i$ex{$i}" class="fancybox-effects-c" title="No.$no&nbsp; ( $name, $sub, ファイル名：$imgtxt{$i} )"><div class="snap"><img src="$cf{imgurl}/$no-$i$ex{$i}" width="$w" height="$h" class="snapshot"></a><br />$imgtxt{$i}</div>|;
		}
	}
	return $ret;
}

#-----------------------------------------------------------
#  コメント返信画像表示(アルバム)
#-----------------------------------------------------------
sub res_image {
	my ($no,$ent,$ex,$w,$h) = @_;

	my $image;
	if (-f "$cf{r_upldir}/$no-s$ex") {
		$image = qq|<img src="$cf{r_upldir}/$no-s$ex" border="0">|;

	} else {
		($w,$h) = resize($w,$h);
		$image = qq|<img src="$cf{r_upldir}/$no$ex" width="$w" height="$h" border="0">|;
	}
	return qq|<a class="fancybox-buttons" data-fancybox-group="button" href="$cf{r_upldir}/$no$ex" title="">$image</a>\n|;
}

#-----------------------------------------------------------
#  取り扱いファイル
#-----------------------------------------------------------
sub ext_file {
	my $ext;
	foreach ( keys %cf ) {
		next if ($_ !~ /^ok_(\w+)/);
		my $file = $1;
		next if (!$cf{$_});
		
		$file =~ tr/a-z/A-Z/;
		$ext .= "$file, ";
	}
	$ext =~ s/, $//;
	
	return $ext;
}

#-----------------------------------------------------------
#  繰越ボタン作成
#-----------------------------------------------------------
sub make_btn {
	my ($i,$pg,$param) = @_;
	
	# ページ繰越数定義
	$cf{pg_max} ||= 10;
	my $next = $pg + $cf{pg_max};
	my $back = $pg - $cf{pg_max};
	
	# ページ繰越ボタン作成
	my @pg;
	if ($back >= 0 || $next < $i) {
		my $flg;
		my ($w,$x,$y,$z) = (0,1,0,$i);
		while ($z > 0) {
			if ($pg == $y) {
				$flg++;
				push(@pg,qq!<span class="page active">$x</span>\n!);
			} else {
				push(@pg,qq!<a href="$cf{bbs_cgi}?pg=$y$param" class="page gradient">$x</a>\n!);
			}
			$x++;
			$y += $cf{pg_max};
			$z -= $cf{pg_max};
			
			if ($flg) { $w++; }
			last if ($w >= 5 && @pg >= 10);
		}
	}
	while( @pg >= 11 ) { shift(@pg); }
	my $ret = join('',@pg);
	if ($back >= 0) {
		$ret = qq!<a href="$cf{bbs_cgi}?pg=$back$param" class="page gradient">&laquo;</a>\n! . $ret;
	}
	if ($next < $i) {
		$ret .= qq!<a href="$cf{bbs_cgi}?pg=$next$param" class="page gradient">&raquo;</a>\n!;
	}
	
	# 結果を返す
	return $ret ? qq|<div class="pagination">\n$ret</div>| : '';
}

#-----------------------------------------------------------
#  ファイル容量
#-----------------------------------------------------------
sub get_size {
	my $file = shift;
	
	my $size = -s $file || 0;
	return $size > 1024 ? comma(int($size/1024)) . 'KB' : $size . 'Bytes';
}

#-----------------------------------------------------------
#  桁区切り
#-----------------------------------------------------------
sub comma {
	local($_) = @_;
	
	1 while s/(.*\d)(\d\d\d)/$1,$2/;
	$_;
}

#-----------------------------------------------------------
#  記事削除確認
#-----------------------------------------------------------
sub del_form {
	my %log;
	open(IN,"$cf{datadir}/log.cgi") or error("open err: log.cgi");
	while (<IN>) {
		my ($no,$date,$nam,$eml,$sub,$com,$url,$hos,$pw,$ex,$w,$h,$tim,$chk) = split(/<>/);
		if ($in{del} == $no) {
			$log{flg}++;
			$log{num}  = $no;
			$log{sub}  = $sub;
			$log{date} = $date;
			$log{name} = $nam;
			last;
		}
	}
	close(IN);
	
	if (!$log{flg}) { error("該当記事が見つかりません"); }
	
	open(IN,"$cf{tmpldir}/dele.html") or error("open err: dele.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	my $pg = $in{pg} || 0;
	$tmpl =~ s/!(\w+_\w+|cmnurl)!/$cf{$1}/g;
	$tmpl =~ s/!(sub|name|date|num)!/$log{$1}/g;
	$tmpl =~ s/!pg!/$pg/g;
	$tmpl =~ s|!icon:(\w+\.\w+)!|<img src="$cf{cmnurl}/$1" alt="$1" class="icon">|g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

