#!/usr/bin/perl

#┌─────────────────────────────────
#│ CLIP BOARD : regist.cgi - 2022/05/28
#│ copyright (c) kentweb, 1997-2022
#│ https://www.kent-web.com/
#└─────────────────────────────────
# REBUILD(改造)
#┌─────────────────────────────────
#│  ClipBoard v6.21 com-multi v2.0 : regist.cgi - 2023/10/17
#│  Edited by TRANSFORM Since 2012
#│  https://www.transformsite.com/
#└─────────────────────────────────
# モジュール宣言
use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib "./lib";
use CGI::Minimal;

# 設定ファイル認識
require "./init.cgi";
my %cf = set_init();

# データ受理
CGI::Minimal::max_read_size($cf{maxdata});
my $cgi = CGI::Minimal->new;
error('容量オーバー') if ($cgi->truncated);
my %in = parse_form($cgi);

# 処理分岐
if ($in{mode} eq 'regist') { regist(); }
if ($in{mode} eq 'usrdel') { usrdel(); }
if ($in{mode} eq 'usredit') { usredit(); }
if ($in{mode} eq "user_lock") { user_lock(); }

error("不明な処理です");

#-----------------------------------------------------------
#  投稿受付処理
#-----------------------------------------------------------
sub regist {

	if ($in{addform}){ # 追加画像数の変更操作・投稿・返信フォームでの操作

	my ($alldata,$data,$key,$value);
	if ($ENV{'REQUEST_METHOD'} eq 'POST') {
	  read(STDIN, $alldata, $ENV{'CONTENT_LENGTH'});
	} else {
	  $alldata = $ENV{'QUERY_STRING'};
	}
	foreach $data (split(/&/, $alldata)) {
	  ($key, $value) = split(/=/, $data);

	  $value =~ s/\+/ /g;
	  $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack('C', hex($1))/eg;
	  $value =~ s/\t//g;

	  $in{$key} = $value;
	}

		# クッキーを発行
		ref_set_cookie($in{name},$in{email},$in{url},$in{formnum});

		# 投稿フォーム画面に戻る
		if ($ENV{PERLXS} eq "PerlIS") {
			print "HTTP/1.0 302 Temporary Redirection\r\n";
			print "Content-type: text/html\n";
		}
		print "Location: $cf{checge_url}\n\n";
	}

	# チェック
	if ($cf{no_wd}) { no_wd(); }
	if ($cf{jp_wd}) { jp_wd(); }
	if ($cf{urlnum} > 0) { urlnum(); }
	
	# 不要改行カット
	$in{sub}  =~ s|<br>||g;
	$in{name} =~ s|<br>||g;
	$in{pwd}  =~ s|<br>||g;
	$in{captcha} =~ s|<br>||g;
	$in{comment} =~ s|(<br>)+$||g;
	
	# 入力値調整
	if ($in{url} eq 'http://') { $in{url} = ''; }
	$in{sub} ||= "無題";
	
	# フォーム内容をチェック
	my $err;
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($in{email} ne '' && $in{email} !~ /^[\w\.\-]+\@[\w\.\-]+\.[a-zA-Z]{2,6}$/) {
		$err .= "Ｅメールの入力内容が不正です<br>";
	}
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/) {
		$err .= "参照先URLの入力内容が不正です<br>";
	}
	if ($err) { error($err); }
	
	# 投稿キーチェック
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		if ($in{captcha} !~ /^\d{$cf{cap_len}}$/) {
			error("投稿キーが入力不備です。<br>投稿フォームに戻って再読込み後、再入力してください");
		}
		
		# 投稿キーチェック
		# -1 : キー不一致
		#  0 : 制限時間オーバー
		#  1 : キー一致
		my $chk = cap::check($in{captcha},$in{str_crypt},$cf{captcha_key},$cf{cap_time},$cf{cap_len});
		if ($chk == 0) {
			error("投稿キーが制限時間を超過しました。<br>投稿フォームに戻って再読込み後、指定の数字を再入力してください");
		} elsif ($chk == -1) {
			error("投稿キーが不正です。<br>投稿フォームに戻って再読込み後、再入力してください");
		}
	}
	
	# 時間取得
	$ENV{TZ} = "JST-9";
	my $now = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($now))[1..6];
	my @week = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
				$year+1900,$mon+1,$mday,$week[$wday],$hour,$min);
	
	# ホスト取得
	my ($host,$addr) = get_host();
	
	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	eval "flock(DAT,2);";
	my $log = <DAT>;
	
	my ($no,$nam,$com,$time) = (split(/<>/, $log))[0,2,5,12];
	if ($in{name} eq $nam && $in{comment} eq $com) {
		close(DAT);
		error("二重投稿は禁止です");
	}
	if ($cf{w_regist} && $now - $time < $cf{w_regist}) {
		close(DAT);
		error("連続投稿はもうしばらく時間を置いてからお願い致します");
	}
	
	# 記事No採番
	$no++;
	
	# 画像アップ
	my ($ex1,$w1,$h1,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6)
			= $in{upfile1} || $in{upfile2} || $in{upfile3} || $in{upfile4} || $in{upfile5} || $in{upfile6} ? upload($no) : ();

	
	# 削除キー暗号化
	my $crypt = encrypt($in{pwd}) if ($in{pwd} ne "");
	
	# 最大記事数処理
	my $i = 0;
	my (@log,@past,$key);
	seek(DAT, 0, 0);
	while (<DAT>) {
		$i++;
		my ($no,$dat,$nam,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);
		
		if ($i <= $cf{maxlog} - 1) {
			push(@log,$_);
		} else {
			if ($ex1) {
				unlink("$cf{imgdir}/$no-1$ex1");
				unlink("$cf{imgdir}/$no-s-1$ex1") if (-f "$cf{imgdir}/$no-s-1$ex1");
			}
			if ($ex2) {
				unlink("$cf{imgdir}/$no-2$ex2");
				unlink("$cf{imgdir}/$no-s-2$ex2") if (-f "$cf{imgdir}/$no-s-2$ex2");
			}
			if ($ex3) {
				unlink("$cf{imgdir}/$no-3$ex3");
				unlink("$cf{imgdir}/$no-s-3$ex3") if (-f "$cf{imgdir}/$no-s-3$ex3");
			}
			if ($ex4) {
				unlink("$cf{imgdir}/$no-4$ex4");
				unlink("$cf{imgdir}/$no-s-4$ex4") if (-f "$cf{imgdir}/$no-s-4$ex4");
			}
			if ($ex5) {
				unlink("$cf{imgdir}/$no-5$ex5");
				unlink("$cf{imgdir}/$no-s-5$ex5") if (-f "$cf{imgdir}/$no-s-5$ex5");
			}
			if ($ex6) {
				unlink("$cf{imgdir}/$no-6$ex6");
				unlink("$cf{imgdir}/$no-s-6$ex6") if (-f "$cf{imgdir}/$no-s-6$ex6");
			}
			# 過去ログ
			if ($cf{pastkey}) { push(@past,$_); }
		}
	}
		# 過去ログ更新
		if (@past > 0) { make_past(@past); }

		if ($in{lockkey}) {
			$key = 1;
		} else {
			$key = 0;
		}
	
	# 新記事
	unshift(@log,"$no<>$date<>$in{name}<>$in{email}<>$in{sub}<>$in{comment}<>$in{url}<>$host<>$crypt<>$ex1<>$w1<>$h1<>$now<>0<>$in{color}<>$in{color2}<>$ex2<>$w2<>$h2<>$ex3<>$w3<>$h3<>$ex4<>$w4<>$h4<>$ex5<>$w5<>$h5<>$ex6<>$w6<>$h6<>$in{imgtext1}<>$in{imgtext2}<>$in{imgtext3}<>$in{imgtext4}<>$in{imgtext5}<>$in{imgtext6}<>$key<>\n");
	
	# 更新
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);

	# レスカウントレイズ
	rescnt_raise();
	
	# クッキー格納
	set_cookie($in{name},$in{email},$in{url},$in{color},$in{color2}) if ($in{cookie} == 1);
	
	# メール通知
	my $clip1 = "$no-1$ex1" if ($ex1);
	my $clip2 = "$no-2$ex2" if ($ex2);
	my $clip3 = "$no-3$ex3" if ($ex3);
	my $clip4 = "$no-4$ex4" if ($ex4);
	my $clip5 = "$no-5$ex5" if ($ex5);
	my $clip6 = "$no-6$ex6" if ($ex6);

	mail_to($date,$host,$clip1,$clip2,$clip3,$clip4,$clip5,$clip6) if ($cf{mailing} == 1);
	
	# 完了画面
	message("ありがとうございます。記事を受理しました。");
}

#-------------------------------------------------
#  レスカウントレイズ
#-------------------------------------------------
sub rescnt_raise {
	# 時間取得
	$ENV{TZ} = "JST-9";
	my $now = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($now))[1..6];
	my @week = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
				$year+1900,$mon+1,$mday,$week[$wday],$hour,$min);

	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: $cf{datadir}/log.cgi");
	eval "flock(DAT, 2);";
	my $log = <DAT>;
	my ($no,$date,$name) = (split(/<>/, $log))[0,1,2];

	# データオープン
	my $res = 0;
	my ($flg,@data);
	open(DAT,"+< $cf{rescnt}");
	eval "flock(DAT, 2);";
	while(<DAT>) {
		my ($no,$date,$name,$res) = split(/<>/);

		if ($in{no} == $no) { 
			$flg = 1;
		}
		push(@data,$_);
	}

	# データ更新
	seek(DAT, 0, 0);
	print DAT @data;
	print DAT "$no<>$date<>$in{name}<>$res<>\n";
	truncate(DAT, tell(DAT));
	close(DAT);
}

#-------------------------------------------------
#  TOPレスカウント更新(親記事用INDEX)
#-------------------------------------------------
sub top_rescnt {

	# コメントログ
	my (%res, @data);
	open(DB,"$cf{reslog}");
	while( my $db = <DB> ) {
		my ($no,$ent,$sub,$date,$name,$url,$host,$com,$ex,$w,$h,$chk,$pw) = split(/<>/, $db);
		$res{$ent}++;
	}
	close(DB);

	# 記事ログ
	open(DB,"$cf{datadir}/log.cgi");
	while( my $db = <DB> ) {
		chomp($db);
		my ($no,$date,$name) = split(/<>/, $db);

		if (!defined($res{$no})) { $res{$no} = 0; }
		push(@data,"$no<>$date<>$name<>$res{$no}<>\n");

	}
	close(DB);

	# index更新
	open(DAT,"+< $cf{rescnt}");
	eval "flock(DAT, 2);";
	seek(DAT, 0, 0);
	print DAT @data;
	truncate(DAT, tell(DAT));
	close(DAT);
}

#-----------------------------------------------------------
#  画像アップロード
#-----------------------------------------------------------
sub upload {
	my $no = shift;

	# サムネイル機能
	require './lib/thumb.pl' if ($cf{thumbnail});

	my @ret;
	foreach my $i (1 .. 6) {
		# mimeタイプ
		my $mime = $cgi->param_mime("upfile$i");

		# 拡張子取得
		my $fname = $cgi->param_filename("upfile$i");
		if ($fname =~ /(\.jpe?g|\.png|\.gif|\.pdf|\.txt|\.doc|\.docx|\.xls|\.xlsx|\.ppt|\.pptx|\.zip|\.mp3|\.mp4)$/i) {
			my $ex = $1;
			$ex =~ tr/A-Z/a-z/;
			if ($ex eq '.jpeg') { $ex = '.jpg'; }

			# 整合性チェック
			my $flg;
			if ($cf{ok_gif}) {
				if ($mime =~ /^image\/gif$/i and $ex eq '.gif') { $flg++; }
			}
			if (!$flg and $cf{ok_jpeg}) {
				if ($mime =~ /^image\/p?jpe?g$/i and $ex eq '.jpg') { $flg++; }
			}
			if (!$flg and $cf{ok_png}) {
				if ($mime =~ /^image\/(x-)?png$/i and $ex eq '.png') { $flg++; }
			}
			if (!$flg and $cf{ok_pdf}) {
				if ($mime =~ /^application\/pdf$/i and $ex eq '.pdf') { $flg++; }
			}
			if (!$flg and $cf{ok_zip}) {
				if ($mime =~ /^application\/(x-)?zip(-compressed)?$/i and $ex eq '.zip') { $flg++; }
			}
			if (!$flg and $cf{ok_text}) {
				if ($mime =~ /^text\/plain$/i and $ex =~ /^\.te?xt$/) { $flg++; }
			}
			if (!$flg and $cf{ok_word}) {
				if ($mime =~ /^application\/(vnd\.)?ms-?word$/i and $ex eq '.doc') { $flg++; }
				elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.wordprocessingml\.document$/i and $ex eq '.docx') { $flg++; }
			}
			if (!$flg and $cf{ok_excel}) {
				if ($mime =~ /^application\/(vnd\.)?ms-?excel$/i and $ex eq '.xls') { $flg++; }
				elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.spreadsheetml\.sheet$/i and $ex eq '.xlsx') { $flg++; }
			}
			if (!$flg and $cf{ok_ppt}) {
				if ($mime =~ /^application\/(vnd\.)?ms-?powerpoint$/i and $ex eq '.ppt') { $flg++; }
				elsif ($mime =~ /^application\/vnd\.openxmlformats-officedocument\.presentationml\.presentation$/i and $ex eq '.pptx') { $flg++; }
			}
			if (!$flg and $cf{ok_mp3}) {
				if ($mime =~ /^audio\/mpeg$/i and $ex eq '.mp3') { $flg++; }
				elsif ($mime =~ /^audio\/mp3/i and $ex eq '.mp3') { $flg++; }
				elsif ($mime =~ /^audio\/mpg/i and $ex eq '.mp3') { $flg++; }
			}
			if (!$flg and $cf{ok_mp4}) {
				if ($mime =~ /^video\/mpeg$/i and $ex eq '.mp4') { $flg++; }
				elsif ($mime =~ /^video\/mp4/i and $ex eq '.mp4') { $flg++; }
				elsif ($mime =~ /^video\/mpg/i and $ex eq '.mp4') { $flg++; }
			}
			if (!$flg) { error('このファイルは取り扱いできません'); }

		# 添付ファイル定義
		my $upfile = "$cf{imgdir}/$no-$i$ex";

		# アップロード書き込み
		open(UP,"+> $upfile") or error("up err: $upfile");
		binmode(UP);
		print UP $in{"upfile$i"};
		close(UP);

		# パーミッション付与
		chmod(0666,$upfile);

		# 画像の場合サイズ取得
		my ($flg,$w,$h);
		if ($ex eq ".jpg") { ($w,$h) = j_size($upfile); $flg++; }
		elsif ($ex eq ".gif") { ($w,$h) = g_size($upfile); $flg++; }
		elsif ($ex eq ".png") { ($w,$h) = p_size($upfile); $flg++; }

		# サムネイル作成
		if ($flg && $cf{thumbnail}) {
			($w,$h) = resize($w,$h);
			my $thumb = "$cf{imgdir}/$no-s-$i$ex";
			make_thumb($upfile,$thumb,$w,$h);
		}

		# 拡張子, 横幅, 縦幅 の順
			push(@ret,($ex,$w,$h));
		} else {
			push(@ret,('','',''));
		}
	}
	# 結果を返す
	return @ret;
}

#-----------------------------------------------------------
#  ユーザ記事削除
#-----------------------------------------------------------
sub usrdel {
	# 入力チェック
	$in{no} =~ s/\D//g;
	if ($in{no} eq '' or $in{pwd} eq '') {
		error("削除Noまたは削除キーが入力モレです");
	}
	
	my ($flg,$crypt,$ext,@log);
	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: log.cgi");
	eval "flock(DAT,2);";
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6) = split(/<>/);
		
		if ($in{no} == $no) {
			$flg++;
			$crypt = $pw;

				# 添付ファイル削除
				if ($ex1) {
					unlink("$cf{imgdir}/$no-1$ex1");
					unlink("$cf{imgdir}/$no-s-1$ex1") if (-f "$cf{imgdir}/$no-s-1$ex1");
				}
				if ($ex2) {
					unlink("$cf{imgdir}/$no-2$ex2");
					unlink("$cf{imgdir}/$no-s-2$ex2") if (-f "$cf{imgdir}/$no-s-2$ex2");
				}
				if ($ex3) {
					unlink("$cf{imgdir}/$no-3$ex3");
					unlink("$cf{imgdir}/$no-s-3$ex3") if (-f "$cf{imgdir}/$no-s-3$ex3");
				}
				if ($ex4) {
					unlink("$cf{imgdir}/$no-4$ex4");
					unlink("$cf{imgdir}/$no-s-4$ex4") if (-f "$cf{imgdir}/$no-s-4$ex4");
				}
				if ($ex5) {
					unlink("$cf{imgdir}/$no-5$ex5");
					unlink("$cf{imgdir}/$no-s-5$ex5") if (-f "$cf{imgdir}/$no-s-5$ex5");
				}
				if ($ex6) {
					unlink("$cf{imgdir}/$no-6$ex6");
					unlink("$cf{imgdir}/$no-s-6$ex6") if (-f "$cf{imgdir}/$no-s-6$ex6");
				}
			next;
		}
		push(@log,$_);
	}
	
	if (!$flg or $crypt eq '') {
		close(DAT);
		error("削除キーが設定されていないか、該当記事が見当たりません");
	}
	
	# 削除キーを照合
	if (decrypt($in{pwd},$crypt) != 1) {
		close(DAT);
		error("認証できません");
	}
	
	# ログを更新
	seek(DAT,0,0);
	print DAT @log;
	truncate(DAT,tell(DAT));
	close(DAT);
	
	message("記事を削除しました");
}

#-----------------------------------------------------------
#  ユーザー記事修正
#-----------------------------------------------------------
sub usredit {

	# 入力チェック
	$in{no} =~ s/\D//g;
	if ($in{no} eq '' || $in{pwd} eq '') {
		&error("記事No又は暗証キーが入力されていません");
	}

	# 実行
	if ($in{job} eq "edit") {

	# チェック
	if ($cf{no_wd}) { no_wd(); }
	if ($cf{jp_wd}) { jp_wd(); }
	if ($cf{urlnum} > 0) { urlnum(); }

	# 不要改行カット
	$in{sub}  =~ s/<br>//g;
	$in{name} =~ s/<br>//g;
	$in{pwd}  =~ s/<br>//g;
	$in{captcha} =~ s/<br>//g;
	$in{comment} =~ s/(<br>)+$//g;

	# 入力値調整
	if ($in{url} eq 'http://') { $in{url} = ''; }
	$in{sub} ||= "無題";

	# フォーム内容をチェック
	my $err;
	if ($in{name} eq "") { $err .= "名前が入力されていません<br>"; }
	if ($in{comment} eq "") { $err .= "コメントが入力されていません<br>"; }
	if ($in{email} ne '' && $in{email} !~ /^[\w\.\-]+\@[\w\.\-]+\.[a-zA-Z]{2,}$/) {
		$err .= "Ｅメールの入力内容が不正です<br>";
	}
	if ($in{url} ne '' && $in{url} !~ /^https?:\/\/[\w-.!~*'();\/?:\@&=+\$,%#]+$/) {
		$err .= "参照先URLの入力内容が不正です<br>";
	}
	if ($err) { error($err); }

		# 画像アップ
		my ($e1n,$w1n,$h1n,$e2n,$w2n,$h2n,$e3n,$w3n,$h3n,$e4n,$w4n,$h4n,$e5n,$w5n,$h5n,$e6n,$w6n,$h6n);
		if ($in{upfile1} || $in{upfile2} || $in{upfile3} || $in{upfile4} || $in{upfile5} || $in{upfile6}) {
			($e1n,$w1n,$h1n,$e2n,$w2n,$h2n,$e3n,$w3n,$h3n,$e4n,$w4n,$h4n,$e5n,$w5n,$h5n,$e6n,$w6n,$h6n) = upload($in{no});
		}

		my ($flg,@data);
		open(DAT,"+< $cf{datadir}/log.cgi") || &error("Open Error: $cf{datadir}/log.cgi");
		eval "flock(DAT, 2);";
		while (<DAT>) {
			my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lockkey) = split(/<>/);

			if ($in{no} == $no) {
			$flg = 1;

			# 画像削除のとき
				if ($in{del1}) {
					unlink("$cf{imgdir}/$no-1$ex1");
					unlink("$cf{imgdir}/$no-s-1$ex1") if (-f "$cf{imgdir}/$no-s-1$ex1");
					$ex1 = $w1 = $h1 = "";
				}
				if ($in{del2}) {
					unlink("$cf{imgdir}/$no-2$ex2");
					unlink("$cf{imgdir}/$no-s-2$ex2") if (-f "$cf{imgdir}/$no-s-2$ex2");
					$ex2 = $w2 = $h2 = "";
				}
				if ($in{del3}) {
					unlink("$cf{imgdir}/$no-3$ex3");
					unlink("$cf{imgdir}/$no-s-3$ex3") if (-f "$cf{imgdir}/$no-s-3$ex3");
					$ex3 = $w3 = $h3 = "";
				}
				if ($in{del4}) {
					unlink("$cf{imgdir}/$no-4$ex4");
					unlink("$cf{imgdir}/$no-s-4$ex4") if (-f "$cf{imgdir}/$no-s-4$ex4");
					$ex4 = $w4 = $h4 = "";
				}
				if ($in{del5}) {
					unlink("$cf{imgdir}/$no-5$ex5");
					unlink("$cf{imgdir}/$no-s-5$ex5") if (-f "$cf{imgdir}/$no-s-5$ex5");
					$ex5 = $w5 = $h5 = "";
				}
				if ($in{del6}) {
					unlink("$cf{imgdir}/$no-6$ex6");
					unlink("$cf{imgdir}/$no-s-6$ex6") if (-f "$cf{imgdir}/$no-s-6$ex6");
					$ex5 = $w5 = $h5 = "";
				}

				# 画像差替（拡張子が異なる場合）
				if ($e1n && $e1n ne $ex1) {
					unlink("$cf{imgdir}/$no-1$ex1");
					unlink("$cf{imgdir}/$no-s-1$ex1") if (-f "$cf{imgdir}/$no-s-1$ex1");
				}
				if ($e2n && $e2n ne $ex2) {
					unlink("$cf{imgdir}/$no-2$ex2");
					unlink("$cf{imgdir}/$no-s-2$ex2") if (-f "$cf{imgdir}/$no-s-2$ex2");
				}
				if ($e3n && $e3n ne $ex3) {
					unlink("$cf{imgdir}/$no-3$ex3");
					unlink("$cf{imgdir}/$no-s-3$ex3") if (-f "$cf{imgdir}/$no-s-3$ex3");
				}
				if ($e4n && $e4n ne $ex4) {
					unlink("$cf{imgdir}/$no-4$ex4");
					unlink("$cf{imgdir}/$no-s-4$ex4") if (-f "$cf{imgdir}/$no-s-4$ex4");
				}
				if ($e5n && $e5n ne $ex5) {
					unlink("$cf{imgdir}/$no-5$ex5");
					unlink("$cf{imgdir}/$no-s-5$ex5") if (-f "$cf{imgdir}/$no-s-5$ex5");
				}
				if ($e6n && $e6n ne $ex6) {
					unlink("$cf{imgdir}/$no-6$ex6");
					unlink("$cf{imgdir}/$no-s-6$ex6") if (-f "$cf{imgdir}/$no-s-6$ex6");
				}

				# 画像差替
				if ($e1n) { $ex1 = $e1n; $w1 = $w1n; $h1 = $h1n; }
				if ($e2n) { $ex2 = $e2n; $w2 = $w2n; $h2 = $h2n; }
				if ($e3n) { $ex3 = $e3n; $w3 = $w3n; $h3 = $h3n; }
				if ($e4n) { $ex4 = $e4n; $w4 = $w4n; $h4 = $h4n; }
				if ($e5n) { $ex5 = $e5n; $w5 = $w5n; $h5 = $h5n; }
				if ($e6n) { $ex6 = $e6n; $w6 = $w6n; $h6 = $h6n; }

				$_ = "$no<>$date<>$in{name}<>$in{email}<>$in{sub}<>$in{comment}<>$in{url}<>$host<>$pw<>$ex1<>$w1<>$h1<>$tim<>$chk<>$in{color}<>$in{color2}<>$ex2<>$w2<>$h2<>$ex3<>$w3<>$h3<>$ex4<>$w4<>$h4<>$ex5<>$w5<>$h5<>$ex6<>$w6<>$h6<>$in{imgtext1}<>$in{imgtext2}<>$in{imgtext3}<>$in{imgtext4}<>$in{imgtext5}<>$in{imgtext6}<>$in{lockkey}<>\n";
			}
			push(@data,$_);
		}

		if ($flg != 1) {
			close(DAT);
			error("不正な処理です");
		}

		# 更新
		seek(DAT, 0, 0);
		print DAT @data;
		truncate(DAT, tell(DAT));
		close(DAT);

		# 完了メッセージ
		message("記事の修正を完了しました");
	}

	my ($flg, @edit);
	open(IN,"$cf{datadir}/log.cgi") || &error("Open Error: $cf{datadir}/log.cgi");
	while (<IN>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lockkey) = split(/<>/);

		if ($in{no} == $no) {

			# 暗証キーなし
			if ($pw eq '') {
				$flg = 2;
			} elsif

			# 暗証キー一致
			(decrypt($in{pwd}, $pw) == 1) {
				$flg = 1;
				@edit = ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lockkey);

			# 暗証キー不一致
			} else {
				$flg = 3;
			}
			last;
		}
	}
	close(IN);

	if (!$flg) {
		close(DAT);
		error("該当記事が見当たりません");
	} elsif ($flg == 2) {
		close(DAT);
		error("この記事には暗証キーが設定されていません");
	} elsif ($flg == 3) {
		close(DAT);
		error("暗証キーが違います");
	}

	# 改行復元
	my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lockkey) = split(/<>/);
	$com =~ s|<br( /)?>|\n|g;
	$url ||= 'http://';

	# 画像認証作成
	my ($str_plain,$str_crypt,$regikey,$JavaScript);
	if ($cf{use_captcha} > 0) {
		require $cf{captcha_pl};
		($str_plain, $str_crypt, $regikey, $JavaScript) = cap::make( $cf{captcha_key}, $cf{cap_len} );
		$JavaScript=&mkJavaScript($regikey,'myFORM') if($cf{cap_len} == 3);
	}

	# 添付ファイル
	my ($attach1,$attach2,$attach3,$attach4,$attach5,$attach6);
	my %e = (1 => $ex1, 2 => $ex2, 3 => $ex3, 4 => $ex4, 5 => $ex5, 6 => $ex6);

	if ($ex1) {
		$attach1 .= qq|<div id="attachment">添付 １ : <label><input type="file" name="upfile1" id="upload1" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach1 .= qq|<input type="text" name="imgtext1" class="reg-imgtxt" placeholder="タイトル" value="$imgtxt1" size="40">\n|;

		$attach1 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-1$ex1" target="_blank">添付</a>]\n|;
		$attach1 .= qq|<input type="checkbox" name="del1" value="1">削除[1]</div>\n|;
	} else {
		$attach1 .= qq|<div id="attachment">添付 １ : <label><input type="file" name="upfile1" id="upload1" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp\n|;
		$attach1 .= qq|<input type="text" name="imgtext1" class="reg-imgtxt" placeholder="タイトル" value="" size="40"></div>\n|;
	}

	if ($ex2) {
		$attach2 .= qq|<div id="attachment">添付 ２ : <label><input type="file" name="upfile2" id="upload2" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach2 .= qq|<input type="text" name="imgtext2" class="reg-imgtxt" placeholder="追加添付１タイトル" value="$imgtxt2" size="40">\n|;

		$attach2 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-2$ex2" target="_blank">添付2</a>]\n|;
		$attach2 .= qq|<input type="checkbox" name="del2" value="1">削除[2]</div><hr><br />\n|;
	} else {
		$attach2 .= qq|<div id="attachment">添付 ２ : <label><input type="file" name="upfile2" id="upload2" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach2 .= qq|<input type="text" name="imgtext2" class="reg-imgtxt" placeholder="追加添付１タイトル" value="" size="40"></div><hr>\n|;
	}

	if ($ex3) {
		$attach3 .= qq|<div id="attachment">添付 ３ : <label><input type="file" name="upfile3" id="upload3" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach3 .= qq|<input type="text" name="imgtext3" class="reg-imgtxt" placeholder="追加添付２タイトル" value="$imgtxt3" size="40">\n|;

		$attach3 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-3$ex3" target="_blank">添付3</a>]\n|;
		$attach3 .= qq|<input type="checkbox" name="del3" value="1">削除[3]</div><hr><br />\n|;
	} else {
		$attach3 .= qq|<div id="attachment">添付 ３ : <label><input type="file" name="upfile3" id="upload3" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach3 .= qq|<input type="text" name="imgtext3" class="reg-imgtxt" placeholder="追加添付２タイトル" value="" size="40"></div><hr>\n|;
	}

	if ($ex4) {
		$attach4 .= qq|<div id="attachment">添付 ４ : <label><input type="file" name="upfile4" id="upload4" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach4 .= qq|<input type="text" name="imgtext4" class="reg-imgtxt" placeholder="追加添付３タイトル" value="$imgtxt4" size="40">\n|;

		$attach4 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-4$ex4" target="_blank">添付4</a>]\n|;
		$attach4 .= qq|<input type="checkbox" name="del4" value="1">削除[4]</div><hr><br />\n|;
	} else {
		$attach4 .= qq|<div id="attachment">添付 ４ : <label><input type="file" name="upfile4" id="upload4" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach4 .= qq|<input type="text" name="imgtext4" class="reg-imgtxt" placeholder="追加添付３タイトル" value="" size="40"></div><hr>\n|;
	}

	if ($ex5) {
		$attach5 .= qq|<div id="attachment">添付 ５ : <label><input type="file" name="upfile5" id="upload5" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach5 .= qq|<input type="text" name="imgtext5" class="reg-imgtxt" placeholder="追加添付４タイトル" value="$imgtxt5" size="40">\n|;

		$attach5 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-5$ex5" target="_blank">添付5</a>]\n|;
		$attach5 .= qq|<input type="checkbox" name="del5" value="1">削除[5]</div><hr><br />\n|;
	} else {
		$attach5 .= qq|<div id="attachment">添付 ５ : <label><input type="file" name="upfile5" id="upload5" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach5 .= qq|<input type="text" name="imgtext5" class="reg-imgtxt" placeholder="追加添付４タイトル" value="" size="40"></div><hr>\n|;
	}

	if ($ex6) {
		$attach6 .= qq|<div id="attachment">添付 ６ : <label><input type="file" name="upfile6" id="upload6" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach6 .= qq|<input type="text" name="imgtext6" class="reg-imgtxt" placeholder="追加添付５タイトル" value="$imgtxt6" size="40">\n|;

		$attach6 .= qq|&nbsp; [<a href="$cf{imgdir}/$no-6$ex6" target="_blank">添付6</a>]\n|;
		$attach6 .= qq|<input type="checkbox" name="del6" value="1">削除[6]</div><hr><br />\n|;
	} else {
		$attach6 .= qq|<div id="attachment">添付 ６ : <label><input type="file" name="upfile6" id="upload6" class="fileinput">ファイルを添付する</label><span class="filename">選択されていません</span>&nbsp;\n|;
		$attach6 .= qq|<input type="text" name="imgtext6" class="reg-imgtxt" placeholder="追加添付５タイトル" value="" size="40"></div>\n|;
	}

	# 色選択ボタン
	my @col = split(/\s+/, $cf{colors});
	my $color;
	foreach (0 .. $#col) {
		if ($_ == $col) {
			$color .= qq|<input type="radio" name="color" value="$_" checked>|;
		} else {
			$color .= qq|<input type="radio" name="color" value="$_">|;
		}
		$color .= qq|<span style="color:$col[$_]">●</span>\n|;
	}

	# 枠色プルダウン
        my @col3 = split(/\s+/, $cf{colors_b});
        my @col4 = split(/\s+/, $cf{colors_b2});
	my $color_b;
	foreach (0 .. $#col3) {
		if ($col3[$_] eq $col2) {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]" selected>$col4[$_]</option>\n|;
		} else {
			$color_b .= qq|<option value="$col3[$_]" style="color:$col3[$_]">$col4[$_]</option>\n|;
		}
	}

	# 修正フォーム
	open(IN,"$cf{tmpldir}/edit.html") or &error("open err: edit.html");
	print "Content-type: text/html; charset=utf-8\n\n";
	while(<IN>) {
                s/!str_crypt!/$str_crypt/g;
		s/!([a-z]+_cgi)!/$cf{$1}/g;
		s/!bbs_title!/$cf{bbs_title}/g;
		s|!bbs_css!|$cf{cmnurl}/bbs.css|g;
		s|!bbs_js!|$cf{cmnurl}/bbs.js|g;
		s/!num!/$no/g;
		s/!pwd!/$pw/g;
		s/!name!/$name/g;
		s/!email!/$eml/g;
		s/!sub!/$sub/g;
		s/!comment!/$com/g;
		s/!color!/$color/g;
		s/<!-- op_color2 -->/$color_b/g;
		s/!col!/$col[$col]/g;
		s/!color2!/$col2/g;
		s/!url!/$url/g;
		s/<!-- attach:1 -->/$attach1/g;
		s/<!-- attach:2 -->/$attach2/g;
		s/<!-- attach:3 -->/$attach3/g;
		s/<!-- attach:4 -->/$attach4/g;
		s/<!-- attach:5 -->/$attach5/g;
		s/<!-- attach:6 -->/$attach6/g;
		print;
	}
	close(IN);

	# フッタ
	footer();

	exit;
}

#-----------------------------------------------------------
#  ユーザースレッドロック開閉
#-----------------------------------------------------------
sub user_lock {

	$in{no} =~ s/\D//g;

	if ($in{no} eq '' or $in{pwd} eq '') {
		error("記事Noまたは暗証キーが入力モレです");
	}

	# ロック情報をマッチングし更新
	my (@new,$key);
	open(DAT,"+< $cf{datadir}/log.cgi") or error("open err: log.cgi");
	eval "flock(DAT,2);";
	while (<DAT>) {
		my ($no,$date,$name,$eml,$sub,$com,$url,$host,$pw,$ex1,$w1,$h1,$tim,$chk,$col,$col2,$ex2,$w2,$h2,$ex3,$w3,$h3,$ex4,$w4,$h4,$ex5,$w5,$h5,$ex6,$w6,$h6,$imgtxt1,$imgtxt2,$imgtxt3,$imgtxt4,$imgtxt5,$imgtxt6,$lock) = split(/<>/);

			if ($in{no} == $no) {

				if ($lock == 0 || $lock == '') {
					$lock = 1;
				} else {
					$lock = 0;
				}
			$key = $lock;

			$_ = "$no<>$date<>$name<>$eml<>$sub<>$com<>$url<>$host<>$pw<>$ex1<>$w1<>$h1<>$tim<>$chk<>$col<>$col2<>$ex2<>$w2<>$h2<>$ex3<>$w3<>$h3<>$ex4<>$w4<>$h4<>$ex5<>$w5<>$h5<>$ex6<>$w6<>$h6<>$imgtxt1<>$imgtxt2<>$imgtxt3<>$imgtxt4<>$imgtxt5<>$imgtxt6<>$lock<>\n";
		}
		push(@new,$_);
	}

	# 更新
	seek(DAT, 0, 0);
	print DAT @new;
	truncate(DAT, tell(DAT));
	close(DAT);

	# 完了画面
	message("スレッドをロックしました。") if ($key eq '1');
	message("スレッドロックを解除しました。") if ($key eq '0');
}

#-----------------------------------------------------------
#  過去ログ生成
#-----------------------------------------------------------
sub make_past {
	my @past = @_;

	# 過去ログNOファイル
	open(NO,"+< $cf{nofile}") or error("open err: $cf{nofile}");
	eval "flock(NO, 2);";
	my $num = <NO>;

	# 過去ログを定義
	my $pastfile = "$cf{pastdir}/" . sprintf("%04d",$num) . ".cgi";

	# 過去ログを開く
	open(LOG,"+< $pastfile") or error("open err: $pastfile");
	eval "flock(LOG, 2);";
	my @data = <LOG>;

	# 規定行数オーバー時、次ファイル生成
	if (@data > $cf{pastmax}) {

		# 過去ログを閉じる
		@data = ();
		close(LOG);

		# 過去NO更新
		seek(NO, 0, 0);
		print NO ++$num;
		truncate(NO, tell(NO));
		close(NO);

		$pastfile = "$cf{pastdir}/" . sprintf("%04d",$num) . ".cgi";

		open(LOG,"+> $pastfile");
		eval "flock(LOG, 2);";
		print LOG @past;
		close(LOG);

		chmod(0666, $pastfile);

	# 規定内
	} else {

		# 過去NOファイルを閉じる
		close(NO);

		# 過去ログ更新
		seek(LOG, 0, 0);
		print LOG @past;
		print LOG @data;
		truncate(LOG, tell(LOG));
		close(LOG);
	}
}

#-----------------------------------------------------------
#  メール送信
#-----------------------------------------------------------
sub mail_to {
	my ($date,$host,$clip1,$clip2,$clip3,$clip4,$clip5,$clip6) = @_;
	
	# メールタイトル定義
	require './lib/jacode.pl';
	my $msub = mime_unstructured_header("BBS: $in{sub}");
	
	# コメント内の改行復元
	my $com = $in{comment};
	$com =~ s|<br>|\n|g;
	$com =~ s/&lt;/>/g;
	$com =~ s/&gt;/</g;
	$com =~ s/&quot;/"/g;
	$com =~ s/&amp;/&/g;
	$com =~ s/&#39;/'/g;
	
	# URL情報
	if ($in{url} eq 'http://') { $in{url} = ''; }
	
	# メール本文を定義
	my $mbody = <<EOM;
掲示板に投稿がありました。

投稿日：$date
ホスト：$host

件名  ：$in{sub}
お名前：$in{name}
E-mail：$in{email}
URL   ：$in{url}
添付1  ：$clip1
添付2  ：$clip2
添付3  ：$clip3
添付4  ：$clip4
添付5  ：$clip5
添付6  ：$clip6

$com
EOM

	my $body;
	for my $tmp ( split(/\n/,$mbody) ) {
		jcode::convert(\$tmp,'jis','utf8');
		$body .= "$tmp\n";
	}
	
	# メールアドレスがない場合は管理者メールに置き換え
	$in{email} ||= $cf{mailto};
	
	# sendmailコマンド
	my $scmd = "$cf{sendmail} -t -i";
	if ($cf{sendm_f}) { $scmd .= " -f $in{email}"; }
	
	# 送信
	open(MAIL,"| $scmd") or error("送信失敗");
	print MAIL "To: $cf{mailto}\n";
	print MAIL "From: $in{email}\n";
	print MAIL "Subject: $msub\n";
	print MAIL "MIME-Version: 1.0\n";
	print MAIL "Content-type: text/plain; charset=ISO-2022-JP\n";
	print MAIL "Content-Transfer-Encoding: 7bit\n";
	print MAIL "X-Mailer: $cf{version}\n\n";
	print MAIL "$body\n";
	close(MAIL);
}

#-----------------------------------------------------------
#  JPEGサイズ認識
#-----------------------------------------------------------
sub j_size {
	my $jpg = shift;
	
	my ($h, $w, $t);
	open(IMG,"$jpg");
	binmode(IMG);
	read(IMG, $t, 2);
	while (1) {
		read(IMG, $t, 4);
		my ($m, $c, $l) = unpack("a a n", $t);
		
		if ($m ne "\xFF") {
			$w = $h = 0;
			last;
		} elsif ((ord($c) >= 0xC0) && (ord($c) <= 0xC3)) {
			read(IMG, $t, 5);
			($h, $w) = unpack("xnn", $t);
			last;
		} else {
			read(IMG, $t, ($l - 2));
		}
	}
	close(IMG);
	
	return ($w, $h);
}

#-----------------------------------------------------------
#  GIFサイズ認識
#-----------------------------------------------------------
sub g_size {
	my $gif = shift;
	
	my $data;
	open(IMG,"$gif");
	binmode(IMG);
	sysread(IMG, $data, 10);
	close(IMG);
	
	if ($data =~ /^GIF/) { $data = substr($data, -4); }
	my $w = unpack("v", substr($data, 0, 2));
	my $h = unpack("v", substr($data, 2, 2));
	
	return ($w, $h);
}

#-----------------------------------------------------------
#  PNGサイズ認識
#-----------------------------------------------------------
sub p_size {
	my $png = shift;
	
	my $data;
	open(IMG,"$png");
	binmode(IMG);
	read(IMG, $data, 24);
	close(IMG);
	
	my $w = unpack("N", substr($data, 16, 20));
	my $h = unpack("N", substr($data, 20, 24));
	
	return ($w, $h);
}

#-----------------------------------------------------------
#  禁止ワードチェック
#-----------------------------------------------------------
sub no_wd {
	my $flg;
	foreach ( split(/,/,$cf{no_wd}) ) {
		if (index("$in{name} $in{sub} $in{comment}",$_) >= 0) {
			$flg = 1;
			last;
		}
	}
	if ($flg) { error("禁止ワードが含まれています"); }
}

#-----------------------------------------------------------
#  日本語チェック
#-----------------------------------------------------------
sub jp_wd {
	if ($in{comment} !~ /(?:[\xC0-\xDF][\x80-\xBF]|[\xE0-\xEF][\x80-\xBF]{2}|[\xF0-\xF7][\x80-\xBF]{3})/x) {
		error("メッセージに日本語が含まれていません");
	}
}

#-----------------------------------------------------------
#  URL個数チェック
#-----------------------------------------------------------
sub urlnum {
	my $com = $in{comment};
	my ($num) = ($com =~ s|(https?://)|$1|ig);
	if ($num > $cf{urlnum}) {
		error("コメント中のURLアドレスは最大$cf{urlnum}個までです");
	}
}

#-----------------------------------------------------------
#  アクセス制限
#-----------------------------------------------------------
sub get_host {
	# IP&ホスト取得
	my $host = $ENV{REMOTE_HOST};
	my $addr = $ENV{REMOTE_ADDR};
	
	if ($cf{gethostbyaddr} && ($host eq "" || $host eq $addr)) {
		$host = gethostbyaddr(pack("C4", split(/\./, $addr)), 2);
	}
	
	# IPチェック
	my $flg;
	foreach ( split(/\s+/, $cf{deny_addr}) ) {
		s/\./\\\./g;
		s/\*/\.\*/g;
		
		if ($addr =~ /^$_/i) { $flg = 1; last; }
	}
	if ($flg) {
		error("アクセスを許可されていません");
	
	# ホストチェック
	} elsif ($host) {
		
		foreach ( split(/\s+/, $cf{deny_host}) ) {
			s/\./\\\./g;
			s/\*/\.\*/g;
			
			if ($host =~ /$_$/i) { $flg = 1; last; }
		}
		if ($flg) {
			error("アクセスを許可されていません");
		}
	}
	if ($host eq "") { $host = $addr; }
	return ($host,$addr);
}

#-----------------------------------------------------------
#  crypt暗号
#-----------------------------------------------------------
sub encrypt {
	my $in = shift;
	
	my @wd = ('a' .. 'z', 'A' .. 'Z', 0 .. 9, '.', '/');
	my $salt = $wd[int(rand(@wd))] . $wd[int(rand(@wd))];
	crypt($in, $salt) || crypt ($in, '$1$' . $salt);
}

#-----------------------------------------------------------
#  crypt照合
#-----------------------------------------------------------
sub decrypt {
	my ($in,$dec) = @_;
	
	my $salt = $dec =~ /^\$1\$(.*)\$/ ? $1 : substr($dec, 0, 2);
	if (crypt($in, $salt) eq $dec || crypt($in, '$1$' . $salt) eq $dec) {
		return 1;
	} else {
		return 0;
	}
}

#-----------------------------------------------------------
#  完了メッセージ
#-----------------------------------------------------------
sub message {
	my ($msg) = @_;
	
	open(IN,"$cf{tmpldir}/mesg.html") or error("open err: mesg.html");
	my $tmpl = join('',<IN>);
	close(IN);
	
	$tmpl =~ s/!(bbs_cgi|bbs_title|cmnurl)!/$cf{$1}/g;
	$tmpl =~ s/!message!/$msg/g;
	
	print "Content-type: text/html; charset=utf-8\n\n";
	print $tmpl;
	exit;
}

#-----------------------------------------------------------
#  時間取得
#-----------------------------------------------------------
sub get_time {
	$ENV{TZ} = "JST-9";
	my $time = time;
	my ($min,$hour,$mday,$mon,$year,$wday) = (localtime($time))[1..6];
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;

	# 日時のフォーマット
	my $date = sprintf("%04d/%02d/%02d(%s) %02d:%02d",
			$year+1900,$mon+1,$mday,$week[$wday],$hour,$min);

	return ($date, $time);
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub ref_set_cookie {
	my (@data,$ref_no) = @_;
	
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime(time + 60*24*60*60);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;
	
	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);
	
	# URLエンコード
	my $cook;
	foreach (@data) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cook .= "$_<>";
	}

	if ($ref_no == 0) { $ref_no = "0"; }

	print "Set-Cookie: $cf{cookie_id2}=$cook; expires=$gmt\n";
}

#-----------------------------------------------------------
#  クッキー発行
#-----------------------------------------------------------
sub set_cookie {
	my @data = @_;
	
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,undef,undef) = gmtime(time + 60*24*60*60);
	my @mon  = qw|Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec|;
	my @week = qw|Sun Mon Tue Wed Thu Fri Sat|;
	
	# 時刻フォーマット
	my $gmt = sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",
				$week[$wday],$mday,$mon[$mon],$year+1900,$hour,$min,$sec);
	
	# URLエンコード
	my $cook;
	foreach (@data) {
		s/(\W)/sprintf("%%%02X", unpack("C", $1))/eg;
		$cook .= "$_<>";
	}
	
	print "Set-Cookie: $cf{cookie_id}=$cook; expires=$gmt\n";
}

#-----------------------------------------------------------
#  フッター
#-----------------------------------------------------------
sub footer {
	my $foot = shift;

	# 著作権表記（削除・改変禁止）
	my $copy = <<EOM;
<div id="footer">
	- <a href="http://www.kent-web.com/" target="_top">$cf{version}</a> -<br>
	++ <a href="http://www.transformsite.com/" target="_top">Edited by TRANSFORM</a> ++
</div>
EOM

	if ($foot =~ /(.+)(<\/body[^>]*>.*)/si) {
		print "$1$copy$2\n";
	} else {
		print "$foot$copy\n";
		print "</body></html>\n";
	}
	exit;
}

#-----------------------------------------------------------
#  GIFチェック
#-----------------------------------------------------------
sub check_gif {
	# マジックバイト確認
	if ($in{upfile} !~ /^gif8[79]a/i) { error('このGIF画像は取扱できません!!'); }
}

#-----------------------------------------------------------
#  PNGチェック
#-----------------------------------------------------------
sub check_png {
	# XSS文字
	if ($in{upfile} =~ /script|embed|object|iframe|onclick|onblur|onchange|onmouse|onError|onload|onfocus|onselect|onsubmit|onunload|onreset|onabort|ondblclick|onkey|ondragdrop/i) {
		error('このPNG画像は取扱できません!');
	}
	# マジックバイト確認
	if ($in{upfile} !~ /^\x89PNG/i) { error('このPNG画像は取扱できません!!'); }
}

#-----------------------------------------------------------
#  JPEGチェック
#-----------------------------------------------------------
sub check_jpg {
	# マジックバイト確認
	if ($in{upfile} !~ /\xff\xd8/) { error('このJPEG画像は取扱できません!!'); }
}

#-----------------------------------------------------------
#  mimeエンコード
#  [quote] http://www.din.or.jp/~ohzaki/perl.htm#JP_Base64
#-----------------------------------------------------------
sub mime_unstructured_header {
  my $oldheader = shift;
  jcode::convert(\$oldheader,'euc','utf8');
  my ($header,@words,@wordstmp,$i);
  my $crlf = $oldheader =~ /\n$/;
  $oldheader =~ s/\s+$//;
  @wordstmp = split /\s+/, $oldheader;
  for ($i = 0; $i < $#wordstmp; $i++) {
    if ($wordstmp[$i] !~ /^[\x21-\x7E]+$/ and
	$wordstmp[$i + 1] !~ /^[\x21-\x7E]+$/) {
      $wordstmp[$i + 1] = "$wordstmp[$i] $wordstmp[$i + 1]";
    } else {
      push(@words, $wordstmp[$i]);
    }
  }
  push(@words, $wordstmp[-1]);
  foreach my $word (@words) {
    if ($word =~ /^[\x21-\x7E]+$/) {
      $header =~ /(?:.*\n)*(.*)/;
      if (length($1) + length($word) > 76) {
	$header .= "\n $word";
      } else {
	$header .= $word;
      }
    } else {
      $header = add_encoded_word($word, $header);
    }
    $header =~ /(?:.*\n)*(.*)/;
    if (length($1) == 76) {
      $header .= "\n ";
    } else {
      $header .= ' ';
    }
  }
  $header =~ s/\n? $//mg;
  $crlf ? "$header\n" : $header;
}
sub add_encoded_word {
  my ($str, $line) = @_;
  my $result;
  my $ascii = '[\x00-\x7F]';
  my $twoBytes = '[\x8E\xA1-\xFE][\xA1-\xFE]';
  my $threeBytes = '\x8F[\xA1-\xFE][\xA1-\xFE]';
  while (length($str)) {
    my $target = $str;
    $str = '';
    if (length($line) + 22 +
	($target =~ /^(?:$twoBytes|$threeBytes)/o) * 8 > 76) {
      $line =~ s/[ \t\n\r]*$/\n/;
      $result .= $line;
      $line = ' ';
    }
    while (1) {
      my $encoded = '=?ISO-2022-JP?B?' .
      b64encode(jcode::jis($target,'euc','z')) . '?=';
      if (length($encoded) + length($line) > 76) {
	$target =~ s/($threeBytes|$twoBytes|$ascii)$//o;
	$str = $1 . $str;
      } else {
	$line .= $encoded;
	last;
      }
    }
  }
  $result . $line;
}
# [quote] http://www.tohoho-web.com/perl/encode.htm
sub b64encode {
    my $buf = shift;
    my ($mode,$tmp,$ret);
    my $b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                . "abcdefghijklmnopqrstuvwxyz"
                . "0123456789+/";
	
    $mode = length($buf) % 3;
    if ($mode == 1) { $buf .= "\0\0"; }
    if ($mode == 2) { $buf .= "\0"; }
    $buf =~ s/(...)/{
        $tmp = unpack("B*", $1);
        $tmp =~ s|(......)|substr($b64, ord(pack("B*", "00$1")), 1)|eg;
        $ret .= $tmp;
    }/eg;
    if ($mode == 1) { $ret =~ s/..$/==/; }
    if ($mode == 2) { $ret =~ s/.$/=/; }
    
    return $ret;
}

